@@ -27,38 +27,21 @@
 using namespace zen;
 using namespace rts;
 
+using fff::FfsExitCode;
+
+#ifdef __WXGTK3__ //deprioritize Wayland: see FFS' application.cpp
+    GLOBAL_RUN_ONCE(::gdk_set_allowed_backends("x11,*")); //call *before* gtk_init()
+#endif
 
 IMPLEMENT_APP(Application)
 
 
 namespace
 {
-wxDEFINE_EVENT(EVENT_ENTER_EVENT_LOOP, wxCommandEvent);
-
-
-using fff::FfsExitCode;
-
-void notifyAppError(const std::wstring& msg, FfsExitCode rc)
+void notifyAppError(const std::wstring& msg)
 {
-    //raiseExitCode(exitCode_, rc);
-
-    const std::wstring msgType = [&]
-    {
-        switch (rc)
-        {
-            //*INDENT-OFF*
-            case FfsExitCode::success: break;
-            case FfsExitCode::warning:   return _("Warning");
-            case FfsExitCode::error:     return _("Error");
-            case FfsExitCode::aborted:   return _("Error");
-            case FfsExitCode::exception: return _("An exception occurred");
-            //*INDENT-ON*
-        }
-        assert(false);
-        return std::wstring{};
-    }();
     //error handling strategy unknown and no sync log output available at this point!
-        std::cerr << utfTo<std::string>(msgType + L": " + msg) + '\n';
+        std::cerr << utfTo<std::string>(_("Error") + L": " + msg) + '\n';
     //alternative0: std::wcerr: cannot display non-ASCII at all, so why does it exist???
     //alternative1: wxSafeShowMessage => NO console output on Debian x86, WTF!
     //alternative2: wxMessageBox() => works, but we probably shouldn't block during command line usage
@@ -70,16 +53,24 @@ bool Application::OnInit()
 {
     //do not call wxApp::OnInit() to avoid using wxWidgets command line parser
 
+    initExtraLog([](const ErrorLog& log) //don't call functions depending on global state (which might be destroyed already!)
+    {
+        std::wstring msg;
+        for (const LogEntry& e : log)
+            msg += utfTo<std::wstring>(formatMessage(e));
+        trim(msg);
+        notifyAppError(msg);
+    });
+
     try { imageResourcesInit(appendPath(fff::getResourceDirPath(), Zstr("Icons.zip"))); }
-    catch (const FileError& e) { notifyAppError(e.toString(), FfsExitCode::warning); }
-    //errors are not really critical in this context
+    catch (const FileError& e) { logExtraError(e.toString()); } //not critical in this context
 
     //GTK should already have been initialized by wxWidgets (see \src\gtk\app.cpp:wxApp::Initialize)
 #if GTK_MAJOR_VERSION == 2
     ::gtk_rc_parse(appendPath(fff::getResourceDirPath(), "Gtk2Styles.rc").c_str());
 
     //fix hang on Ubuntu 19.10 (see FFS's application.cpp)
-    g_vfs_get_default(); //returns unowned GVfs*
+    [[maybe_unused]] GVfs* defaultFs = ::g_vfs_get_default(); //not owned by us!
 
 #elif GTK_MAJOR_VERSION == 3
     auto loadCSS = [&](const char* fileName)
@@ -90,14 +81,14 @@ bool Application::OnInit()
         GError* error = nullptr;
         ZEN_ON_SCOPE_EXIT(if (error) ::g_error_free(error));
 
-        ::gtk_css_provider_load_from_path(provider, //GtkCssProvider* css_provider,
-                                          appendPath(fff::getResourceDirPath(), fileName).c_str(), //const gchar* path,
+        ::gtk_css_provider_load_from_path(provider, //GtkCssProvider* css_provider
+                                          appendPath(fff::getResourceDirPath(), fileName).c_str(), //const gchar* path
                                           &error); //GError** error
         if (error)
             throw SysError(formatGlibError("gtk_css_provider_load_from_path", error));
 
-        ::gtk_style_context_add_provider_for_screen(::gdk_screen_get_default(),               //GdkScreen* screen,
-                                                    GTK_STYLE_PROVIDER(provider),             //GtkStyleProvider* provider,
+        ::gtk_style_context_add_provider_for_screen(::gdk_screen_get_default(),               //GdkScreen* screen
+                                                    GTK_STYLE_PROVIDER(provider),             //GtkStyleProvider* provider
                                                     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION); //guint priority
     };
     try
@@ -111,62 +102,58 @@ bool Application::OnInit()
         {
             loadCSS("Gtk3Styles.old.css"); //throw SysError
         }
-        catch (const SysError& e2) { notifyAppError(e2.toString(), FfsExitCode::warning); }
+        catch (const SysError& e3) { logExtraError(_("Error during process initialization.") + L"\n\n" + e3.toString()); }
     }
 #else
 #error unknown GTK version!
 #endif
 
-    try
-    {
-        /* we're a GUI app: ignore SIGHUP when the parent terminal quits! (or process is killed!)
+    /* we're a GUI app: ignore SIGHUP when the parent terminal quits! (or process is killed!)
             => the FFS launcher will still be killed => fine
             => macOS: apparently not needed! interestingly the FFS launcher does receive SIGHUP and *is* killed  */
-        if (sighandler_t oldHandler = ::signal(SIGHUP, SIG_IGN);
-            oldHandler == SIG_ERR)
-            THROW_LAST_SYS_ERROR("signal(SIGHUP)");
-        else assert(!oldHandler);
-    }
-    catch (const SysError& e) { notifyAppError(e.toString(), FfsExitCode::warning); }
+    if (sighandler_t oldHandler = ::signal(SIGHUP, SIG_IGN);
+        oldHandler == SIG_ERR)
+        logExtraError(_("Error during process initialization.") + L"\n\n" + formatSystemError("signal(SIGHUP)", getLastError()));
+    else assert(!oldHandler);
 
 
     //Windows User Experience Interaction Guidelines: tool tips should have 5s timeout, info tips no timeout => compromise:
     wxToolTip::Enable(true); //wxWidgets screw-up: wxToolTip::SetAutoPop is no-op if global tooltip window is not yet constructed: wxToolTip::Enable creates it
-    wxToolTip::SetAutoPop(10'000); //https://docs.microsoft.com/en-us/windows/win32/uxguide/ctrl-tooltips-and-infotips
+    wxToolTip::SetAutoPop(15'000); //https://docs.microsoft.com/en-us/windows/win32/uxguide/ctrl-tooltips-and-infotips
+
+    SetAppName(L"RealTimeSync"); //if not set, defaults to executable name
 
-    SetAppName(L"RealTimeSync");
 
     try
     {
         fff::localizationInit(appendPath(fff::getResourceDirPath(), Zstr("Languages.zip"))); //throw FileError
         fff::setLanguage(getProgramLanguage()); //throw FileError
     }
-    catch (const FileError& e) { notifyAppError(e.toString(), FfsExitCode::warning); }
+    catch (const FileError& e) { logExtraError(e.toString()); }
 
-
-    auto onSystemShutdown = []
+    auto onSystemShutdown = [](int /*unused*/ = 0)
     {
         onSystemShutdownRunTasks();
 
         //it's futile to try and clean up while the process is in full swing (CRASH!) => just terminate!
-        terminateProcess(static_cast<int>(FfsExitCode::aborted));
+        terminateProcess(static_cast<int>(FfsExitCode::cancelled));
     };
     Bind(wxEVT_QUERY_END_SESSION, [onSystemShutdown](wxCloseEvent& event) { onSystemShutdown(); }); //can veto
     Bind(wxEVT_END_SESSION,       [onSystemShutdown](wxCloseEvent& event) { onSystemShutdown(); }); //can *not* veto
+    if (auto /*sighandler_t n.a. on macOS*/ oldHandler = ::signal(SIGTERM, onSystemShutdown);//"graceful" exit requested, unlike SIGKILL
+        oldHandler == SIG_ERR)
+        logExtraError(_("Error during process initialization.") + L"\n\n" + formatSystemError("signal(SIGTERM)", getLastError()));
+    else assert(!oldHandler);
 
     //Note: app start is deferred:  -> see FreeFileSync
-    Bind(EVENT_ENTER_EVENT_LOOP, &Application::onEnterEventLoop, this);
-    wxCommandEvent scrollEvent(EVENT_ENTER_EVENT_LOOP);
-    AddPendingEvent(scrollEvent);
+    CallAfter([&] { onEnterEventLoop(); });
+
     return true; //true: continue processing; false: exit immediately.
 }
 
 
-void Application::onEnterEventLoop(wxEvent& event)
+void Application::onEnterEventLoop()
 {
-    [[maybe_unused]] bool ubOk = Unbind(EVENT_ENTER_EVENT_LOOP, &Application::onEnterEventLoop, this);
-    assert(ubOk);
-
     //wxWidgets app exit handling is weird... we want to exit only if the logical main window is closed, not just *any* window!
     wxTheApp->SetExitOnFrameDelete(false); //prevent popup-windows from becoming temporary top windows leading to program exit after closure
     ZEN_ON_SCOPE_EXIT(if (!globalWindowWasSet()) wxTheApp->ExitMainLoop()); //quit application, if no main window was set (batch silent mode)
@@ -202,7 +189,7 @@ void Application::onEnterEventLoop(wxEvent& event)
     }
     catch (const FileError& e)
     {
-        notifyAppError(e.toString(), FfsExitCode::exception);
+        notifyAppError(e.toString());
     }
 }
 
@@ -235,7 +222,7 @@ void Application::OnUnhandledException() //handles both wxApp::OnInit() + wxApp:
     }
     catch (const std::bad_alloc& e) //the only kind of exception we don't want crash dumps for
     {
-        notifyAppError(utfTo<std::wstring>(e.what()), FfsExitCode::exception);
+        notifyAppError(utfTo<std::wstring>(e.what()));
         terminateProcess(static_cast<int>(FfsExitCode::exception));
     }
     //catch (...) -> Windows: let it crash and create mini dump!!! Linux/macOS: std::exception::what() logged to console
