@@ -9,7 +9,7 @@
 #include <wx/filedlg.h>
 #include <wx+/app_main.h>
 #include <wx+/bitmap_button.h>
-#include <wx+/font_size.h>
+#include <wx+/window_layout.h>
 #include <wx+/popup_dlg.h>
 #include <wx+/image_resources.h>
 #include <zen/file_access.h>
@@ -81,18 +81,18 @@ MainDialog::MainDialog(const Zstring& cfgFilePath) :
     m_scrolledWinFolders->SetScrollRate(scrollDelta, scrollDelta);
 
     m_txtCtrlDirectoryMain->SetMinSize({fastFromDIP(300), -1});
-    m_spinCtrlDelay       ->SetMinSize({fastFromDIP( 70), -1}); //Hack: set size (why does wxWindow::Size() not work?)
+    setDefaultWidth(*m_spinCtrlDelay);
 
     m_bpButtonRemoveTopFolder->Hide();
     m_panelMainFolder->Layout();
 
-    setImage(*m_bitmapBatch,   loadImage("cfg_batch_sicon"));
+    setImage(*m_bitmapBatch,   loadImage("cfg_batch", fastFromDIP(20)));
     setImage(*m_bitmapFolders, fff::IconBuffer::genericDirIcon(fff::IconBuffer::IconSize::small));
     setImage(*m_bitmapConsole, loadImage("command_line", fastFromDIP(20)));
 
-    setImage(*m_bpButtonAddFolder, loadImage("item_add"));
+    setImage(*m_bpButtonAddFolder,       loadImage("item_add"));
     setImage(*m_bpButtonRemoveTopFolder, loadImage("item_remove"));
-    setBitmapTextLabel(*m_buttonStart, loadImage("startRts"), m_buttonStart->GetLabelText(), fastFromDIP(5), fastFromDIP(8));
+    setBitmapTextLabel(*m_buttonStart, loadImage("start_rts"), m_buttonStart->GetLabelText(), fastFromDIP(5), fastFromDIP(8));
 
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); });
 
@@ -101,7 +101,7 @@ MainDialog::MainDialog(const Zstring& cfgFilePath) :
     setGlobalWindow(this);
 
     //prepare drag & drop
-    firstFolderPanel_ = std::make_unique<FolderSelector2>(this, *m_panelMainFolder, *m_buttonSelectFolderMain, *m_txtCtrlDirectoryMain, folderLastSelected_, m_staticTextFinalPath);
+    firstFolderPanel_ = std::make_unique<FolderSelector2>(this, *m_panelMainFolder, *m_buttonSelectFolderMain, *m_txtCtrlDirectoryMain, folderLastSelected_, nullptr /*staticText*/);
 
     //--------------------------- load config values ------------------------------------
     XmlRealConfig newConfig;
@@ -120,7 +120,7 @@ MainDialog::MainDialog(const Zstring& cfgFilePath) :
         try
         {
             std::wstring warningMsg;
-            readRealOrBatchConfig(currentConfigFile, newConfig, warningMsg); //throw FileError
+            std::tie(newConfig, warningMsg) = readRealOrBatchConfig(currentConfigFile); //throw FileError
 
             if (!warningMsg.empty())
                 showNotificationDialog(this, DialogInfoType::warning, PopupDialogCfg().setDetailInstructions(warningMsg));
@@ -176,9 +176,8 @@ MainDialog::~MainDialog()
 
 void MainDialog::onBeforeSystemShutdown()
 {
-    try { writeConfig(getConfiguration(), lastRunConfigPath_); } //throw FileError
-    catch (FileError&) { assert(false); }
-    warn_static("log, maybe?")
+    try { writeConfig(getConfiguration(), lastRunConfigPath_); }
+    catch (const FileError& e) { logExtraError(e.toString()); }
 }
 
 
@@ -225,15 +224,15 @@ void MainDialog::onStart(wxCommandEvent& event)
 
     switch (runFolderMonitor(currentCfg, ::extractJobName(activeCfgFilePath)))
     {
-        case AbortReason::REQUEST_EXIT:
+        case CancelReason::requestExit:
             Close();
             return;
 
-        case AbortReason::REQUEST_GUI:
+        case CancelReason::requestGui:
             break;
     }
 
-    Show(); //don't show for AbortReason::REQUEST_EXIT
+    Show(); //don't show for CancelReason::requestExit
     Raise();
     m_buttonStart->SetFocus();
 }
@@ -283,7 +282,7 @@ void MainDialog::loadConfig(const Zstring& filepath)
         try
         {
             std::wstring warningMsg;
-            readRealOrBatchConfig(filepath, newConfig, warningMsg); //throw FileError
+            std::tie(newConfig, warningMsg) = readRealOrBatchConfig(filepath); //throw FileError
 
             if (!warningMsg.empty())
                 showNotificationDialog(this, DialogInfoType::warning, PopupDialogCfg().setDetailInstructions(warningMsg));
