@@ -698,13 +698,13 @@ std::vector<std::pair<BaseFolderPair*, SyncDirectionConfig>> fff::extractDirecti
                     mainCfg.additionalPairs.end());
 
     if (folderCmp.size() != allPairs.size())
-        throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
+        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
 
     std::vector<std::pair<BaseFolderPair*, SyncDirectionConfig>> output;
 
     for (auto it = folderCmp.begin(); it != folderCmp.end(); ++it)
     {
-        BaseFolderPair& baseFolder = **it;
+        BaseFolderPair& baseFolder = it->ref();
         const LocalPairConfig& lpc = allPairs[it - folderCmp.begin()];
 
         output.emplace_back(&baseFolder, lpc.localSyncCfg ? lpc.localSyncCfg->directionCfg : mainCfg.syncCfg.directionCfg);
@@ -998,7 +998,7 @@ void fff::applyFiltering(FolderComparison& folderCmp, const MainConfiguration& m
     if (folderCmp.empty())
         return;
     else if (folderCmp.size() != mainCfg.additionalPairs.size() + 1)
-        throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
+        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
 
     //merge first and additional pairs
     std::vector<LocalPairConfig> allPairs;
@@ -1009,7 +1009,7 @@ void fff::applyFiltering(FolderComparison& folderCmp, const MainConfiguration& m
 
     for (auto it = allPairs.begin(); it != allPairs.end(); ++it)
     {
-        BaseFolderPair& baseFolder = *folderCmp[it - allPairs.begin()];
+        BaseFolderPair& baseFolder = folderCmp[it - allPairs.begin()].ref();
 
         const NormalizedFilter normFilter = normalizeFilters(mainCfg.globalFilter, it->localFilter);
 
@@ -1468,7 +1468,7 @@ void fff::deleteFromGridAndHD(const std::vector<FileSystemObject*>& rowsToDelete
     //ensure cleanup: redetermination of sync-directions and removal of invalid rows
     auto updateDirection = [&]
     {
-        //update sync direction: we cannot do a full redetermination since the user may already have entered manual changes
+        //update sync direction: we cannot do a full redetermination since the user may have manual changes applied already 
         std::vector<FileSystemObject*> rowsToDelete;
         append(rowsToDelete, deleteLeft);
         append(rowsToDelete, deleteRight);
@@ -1566,8 +1566,7 @@ TempFileBuffer::~TempFileBuffer()
         {
             removeDirectoryPlainRecursion(tempFolderPath_); //throw FileError
         }
-        catch (FileError&) { assert(false); }
-    warn_static("log, maybe?")
+        catch (const FileError& e) { logExtraError(e.toString()); }
 }
 
 
