@@ -8,7 +8,6 @@
 #include <zen/process_priority.h>
 #include <zen/perf.h>
 #include <zen/time.h>
-#include <wx/datetime.h>
 #include "algorithm.h"
 #include "parallel_scan.h"
 #include "dir_exist_async.h"
@@ -76,25 +75,55 @@ ResolvedBaseFolders initializeBaseFolders(const std::vector<FolderPairCfg>& fpCf
                                           WarningDialogs& warnings,
                                           PhaseCallback& callback /*throw X*/) //throw X
 {
+    std::vector<Zstring> pathPhrases;
+    for (const FolderPairCfg& fpCfg : fpCfgList)
+    {
+        pathPhrases.push_back(fpCfg.folderPathPhraseLeft_);
+        pathPhrases.push_back(fpCfg.folderPathPhraseRight_);
+    }
+
     ResolvedBaseFolders output;
     std::set<AbstractPath> allFolders;
 
     tryReportingError([&]
     {
+        //createAbstractPath() -> tryExpandVolumeName() hangs for idle HDD! => run async to make it cancellable
+        auto protCurrentPhrase = makeSharedRef<Protected<Zstring>>();
+
+        std::future<std::vector<AbstractPath>> futFolderPaths = runAsync([pathPhrases, currentPhraseWeak = std::weak_ptr(protCurrentPhrase.ptr())]
+        {
+            setCurrentThreadName(Zstr("Normalizing folder paths"));
+
+            std::vector<AbstractPath> folderPaths;
+            for (const Zstring& pathPhrase : pathPhrases)
+            {
+                if (auto protCurrentPhrase2 = currentPhraseWeak.lock()) //[!] not owned by worker thread!
+                    protCurrentPhrase2->access([&](Zstring& currentPathPhrase) { currentPathPhrase = pathPhrase; });
+                else
+                    throw std::runtime_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Caller context gone!");
+
+                folderPaths.push_back(createAbstractPath(pathPhrase));
+            }
+            return folderPaths;
+        });
+
+        while (futFolderPaths.wait_for(UI_UPDATE_INTERVAL / 2) == std::future_status::timeout)
+        {
+            const Zstring pathPhrase = protCurrentPhrase.ref().access([](const Zstring& currentPathPhrase) { return currentPathPhrase; });
+            callback.updateStatus(_("Normalizing folder paths...") + L' ' + utfTo<std::wstring>(pathPhrase)); //throw X
+        }
+
+        const std::vector<AbstractPath>& folderPaths = futFolderPaths.get(); //throw (std::runtime_error)
+
         //support "retry" for environment variable and variable driver letter resolution!
         allFolders.clear();
-        output.resolvedPairs.clear();
-        for (const FolderPairCfg& fpCfg : fpCfgList)
-        {
-            output.resolvedPairs.push_back(
-            {
-                createAbstractPath(fpCfg.folderPathPhraseLeft_),
-                createAbstractPath(fpCfg.folderPathPhraseRight_)});
+        allFolders.insert(folderPaths.begin(), folderPaths.end());
 
-            allFolders.insert(output.resolvedPairs.back().folderPathLeft);
-            allFolders.insert(output.resolvedPairs.back().folderPathRight);
-        }
+        output.resolvedPairs.clear();
+        for (size_t i = 0; i < folderPaths.size(); i += 2)
+            output.resolvedPairs.push_back({folderPaths[i], folderPaths[i + 1]});
         //---------------------------------------------------------------------------
+
         output.baseFolderStatus = getFolderStatusParallel(allFolders,
                                                           true /*authenticateAccess*/, requestPassword, callback); //throw X
         if (!output.baseFolderStatus.failedChecks.empty())
@@ -157,44 +186,61 @@ ResolvedBaseFolders initializeBaseFolders(const std::vector<FolderPairCfg>& fpCf
 class ComparisonBuffer
 {
 public:
-    ComparisonBuffer(const std::set<DirectoryKey>& folderKeys,
-                     const FolderStatus& baseFolderStatus,
+    ComparisonBuffer(const FolderStatus& folderStatus,
                      int fileTimeTolerance,
-                     ProcessCallback& callback);
+                     ProcessCallback& callback) :
+        fileTimeTolerance_(fileTimeTolerance),
+        folderStatus_(folderStatus),
+        cb_(callback) {}
 
-    //create comparison result table and fill category except for files existing on both sides: undefinedFiles and undefinedSymlinks are appended!
-    std::shared_ptr<BaseFolderPair> compareByTimeSize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const;
-    std::shared_ptr<BaseFolderPair> compareBySize    (const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const;
-    std::vector<std::shared_ptr<BaseFolderPair>> compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const;
+    FolderComparison execute(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad);
 
 private:
     ComparisonBuffer           (const ComparisonBuffer&) = delete;
     ComparisonBuffer& operator=(const ComparisonBuffer&) = delete;
 
-    std::shared_ptr<BaseFolderPair> performComparison(const ResolvedFolderPair& fp,
-                                                      const FolderPairCfg& fpCfg,
-                                                      std::vector<FilePair*>& undefinedFiles,
-                                                      std::vector<SymlinkPair*>& undefinedSymlinks) const;
+    //create comparison result table and fill category except for files existing on both sides: undefinedFiles and undefinedSymlinks are appended!
+    SharedRef<BaseFolderPair> compareByTimeSize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const;
+    SharedRef<BaseFolderPair> compareBySize    (const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const;
+    std::vector<SharedRef<BaseFolderPair>> compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const;
+
+    SharedRef<BaseFolderPair> performComparison(const ResolvedFolderPair& fp,
+                                                const FolderPairCfg& fpCfg,
+                                                std::vector<FilePair*>& undefinedFiles,
+                                                std::vector<SymlinkPair*>& undefinedSymlinks) const;
+
+    BaseFolderStatus getBaseFolderStatus(const AbstractPath& folderPath) const
+    {
+        if (folderStatus_.existing.contains(folderPath))
+            return BaseFolderStatus::existing;
+        if (folderStatus_.notExisting.contains(folderPath))
+            return BaseFolderStatus::notExisting;
+        if (folderStatus_.failedChecks.contains(folderPath))
+            return BaseFolderStatus::failure;
+        assert(AFS::isNullPath(folderPath));
+        return BaseFolderStatus::notExisting;
+    };
 
-    std::map<DirectoryKey, DirectoryValue> folderBuffer_; //contains entries for *all* scanned folders!
     const int fileTimeTolerance_;
     const FolderStatus& folderStatus_;
+    std::map<DirectoryKey, DirectoryValue> folderBuffer_; //contains entries for *all* scanned folders!
     ProcessCallback& cb_;
 };
 
 
-ComparisonBuffer::ComparisonBuffer(const std::set<DirectoryKey>& folderKeys,
-                                   const FolderStatus& folderStatus,
-                                   int fileTimeTolerance,
-                                   ProcessCallback& callback) :
-    fileTimeTolerance_(fileTimeTolerance),
-    folderStatus_(folderStatus),
-    cb_(callback)
+FolderComparison ComparisonBuffer::execute(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad)
 {
     std::set<DirectoryKey> foldersToRead;
-    for (const DirectoryKey& folderKey : folderKeys)
-        if (folderStatus.existing.contains(folderKey.folderPath))
-            foldersToRead.insert(folderKey); //only traverse *existing* folders
+    for (const auto& [folderPair, fpCfg] : workLoad)
+        if (getBaseFolderStatus(folderPair.folderPathLeft ) != BaseFolderStatus::failure && //no need to list or display one-sided results if
+            getBaseFolderStatus(folderPair.folderPathRight) != BaseFolderStatus::failure)   //*either* folder existence check fails
+        {
+            //+ only traverse *existing* folders
+            if (getBaseFolderStatus(folderPair.folderPathLeft) == BaseFolderStatus::existing)
+                foldersToRead.emplace(DirectoryKey({folderPair.folderPathLeft,  fpCfg.filter.nameFilter, fpCfg.handleSymlinks}));
+            if (getBaseFolderStatus(folderPair.folderPathRight) == BaseFolderStatus::existing)
+                foldersToRead.emplace(DirectoryKey({folderPair.folderPathRight, fpCfg.filter.nameFilter, fpCfg.handleSymlinks}));
+        }
 
     //------------------------------------------------------------------
     const std::chrono::steady_clock::time_point compareStartTime = std::chrono::steady_clock::now();
@@ -202,38 +248,54 @@ ComparisonBuffer::ComparisonBuffer(const std::set<DirectoryKey>& folderKeys,
 
     auto onStatusUpdate = [&, textScanning = _("Scanning:") + L' '](const std::wstring& statusLine, int itemsTotal)
     {
-        callback.updateDataProcessed(itemsTotal - itemsReported, 0); //noexcept
+        cb_.updateDataProcessed(itemsTotal - itemsReported, 0); //noexcept
         itemsReported = itemsTotal;
 
-        callback.updateStatus(textScanning + statusLine); //throw X
+        cb_.updateStatus(textScanning + statusLine); //throw X
     };
 
+    //PERF_START;
     folderBuffer_ = parallelDeviceTraversal(foldersToRead,
-    [&](const PhaseCallback::ErrorInfo& errorInfo) { return callback.reportError(errorInfo); }, //throw X
+    [&](const PhaseCallback::ErrorInfo& errorInfo) { return cb_.reportError(errorInfo); }, //throw X
     onStatusUpdate, //throw X
     UI_UPDATE_INTERVAL / 2); //every ~50 ms
+    //PERF_STOP;
 
     const int64_t totalTimeSec = std::chrono::duration_cast<std::chrono::seconds>(std::chrono::steady_clock::now() - compareStartTime).count();
-
-    callback.logMessage(_("Comparison finished:") + L' ' +
-                        _P("1 item found", "%x items found", itemsReported) + SPACED_DASH +
-                        _("Time elapsed:") + L' ' + copyStringTo<std::wstring>(wxTimeSpan::Seconds(totalTimeSec).Format()),
-                        PhaseCallback::MsgType::info); //throw X
+    cb_.logMessage(_("Comparison finished:") + L' ' +
+                   _P("1 item found", "%x items found", itemsReported) + SPACED_DASH +
+                   _("Time elapsed:") + L' ' + utfTo<std::wstring>(formatTimeSpan(totalTimeSec)),
+                   PhaseCallback::MsgType::info); //throw X
     //------------------------------------------------------------------
 
-    //folderStatus_.existing already in buffer, now create entries for the rest:
-    for (const DirectoryKey& folderKey : folderKeys)
-        if (auto it = folderStatus_.failedChecks.find(folderKey.folderPath);
-            it != folderStatus_.failedChecks.end())
-            //make sure all items are disabled => avoid user panicking: https://freefilesync.org/forum/viewtopic.php?t=7582
-            folderBuffer_[folderKey].failedFolderReads[Zstring() /*empty string for root*/] = utfTo<Zstringc>(it->second.toString());
-        else
+    //process binary comparison as one junk
+    std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>> workLoadByContent;
+    for (const auto& [folderPair, fpCfg] : workLoad)
+        if (fpCfg.compareVar == CompareVariant::content)
+            workLoadByContent.push_back({folderPair, fpCfg});
+
+    std::vector<SharedRef<BaseFolderPair>> outputByContent = compareByContent(workLoadByContent);
+    auto itOByC = outputByContent.begin();
+
+    FolderComparison output;
+
+    //write output in expected order
+    for (const auto& [folderPair, fpCfg] : workLoad)
+        switch (fpCfg.compareVar)
         {
-            folderBuffer_[folderKey];
-            assert(folderStatus_.existing   .contains(folderKey.folderPath) ||
-                   folderStatus_.notExisting.contains(folderKey.folderPath) ||
-                   AFS::isNullPath(folderKey.folderPath));
+            case CompareVariant::timeSize:
+                output.push_back(compareByTimeSize(folderPair, fpCfg));
+                break;
+            case CompareVariant::size:
+                output.push_back(compareBySize(folderPair, fpCfg));
+                break;
+            case CompareVariant::content:
+                assert(itOByC != outputByContent.end());
+                if (itOByC != outputByContent.end())
+                    output.push_back(*itOByC++);
+                break;
         }
+    return output;
 }
 
 
@@ -332,12 +394,12 @@ void categorizeSymlinkByTime(SymlinkPair& symlink)
 }
 
 
-std::shared_ptr<BaseFolderPair> ComparisonBuffer::compareByTimeSize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const
+SharedRef<BaseFolderPair> ComparisonBuffer::compareByTimeSize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const
 {
     //do basis scan and retrieve files existing on both sides as "compareCandidates"
     std::vector<FilePair*> uncategorizedFiles;
     std::vector<SymlinkPair*> uncategorizedLinks;
-    std::shared_ptr<BaseFolderPair> output = performComparison(fp, fpConfig, uncategorizedFiles, uncategorizedLinks);
+    SharedRef<BaseFolderPair> output = performComparison(fp, fpConfig, uncategorizedFiles, uncategorizedLinks);
 
     //finish symlink categorization
     for (SymlinkPair* symlink : uncategorizedLinks)
@@ -428,12 +490,12 @@ void categorizeSymlinkByContent(SymlinkPair& symlink, PhaseCallback& callback)
 }
 
 
-std::shared_ptr<BaseFolderPair> ComparisonBuffer::compareBySize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const
+SharedRef<BaseFolderPair> ComparisonBuffer::compareBySize(const ResolvedFolderPair& fp, const FolderPairCfg& fpConfig) const
 {
     //do basis scan and retrieve files existing on both sides as "compareCandidates"
     std::vector<FilePair*> uncategorizedFiles;
     std::vector<SymlinkPair*> uncategorizedLinks;
-    std::shared_ptr<BaseFolderPair> output = performComparison(fp, fpConfig, uncategorizedFiles, uncategorizedLinks);
+    SharedRef<BaseFolderPair> output = performComparison(fp, fpConfig, uncategorizedFiles, uncategorizedLinks);
 
     //finish symlink categorization
     for (SymlinkPair* symlink : uncategorizedLinks)
@@ -529,7 +591,7 @@ void categorizeFileByContent(FilePair& file, const std::wstring& txtComparingCon
 }
 
 
-std::vector<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const
+std::vector<SharedRef<BaseFolderPair>> ComparisonBuffer::compareByContent(const std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>>& workLoad) const
 {
     struct ParallelOps
     {
@@ -552,7 +614,7 @@ std::vector<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(
         fpWorkload.push_back({posL, posR, std::move(filesToCompareBytewise)});
     };
 
-    std::vector<std::shared_ptr<BaseFolderPair>> output;
+    std::vector<SharedRef<BaseFolderPair>> output;
 
     const Zstringc txtConflictSkippedBinaryComparison = getConflictSkippedBinaryComparison(); //avoid premature pess.: save memory via ref-counted string
 
@@ -580,8 +642,8 @@ std::vector<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(
                     filesToCompareBytewise.push_back(file);
             }
         if (!filesToCompareBytewise.empty())
-            addToBinaryWorkload(output.back()->getAbstractPath<SelectSide::left >(),
-                                output.back()->getAbstractPath<SelectSide::right>(), std::move(filesToCompareBytewise));
+            addToBinaryWorkload(output.back().ref().getAbstractPath<SelectSide::left >(),
+                                output.back().ref().getAbstractPath<SelectSide::right>(), std::move(filesToCompareBytewise));
 
         //finish symlink categorization
         for (SymlinkPair* symlink : uncategorizedLinks)
@@ -589,18 +651,18 @@ std::vector<std::shared_ptr<BaseFolderPair>> ComparisonBuffer::compareByContent(
     }
 
     //finish categorization: compare files (that have same size) bytewise...
-    if (!fpWorkload.empty()) //run ProcessPhase::comparingContent only when needed
+    if (!fpWorkload.empty()) //run ProcessPhase::binaryCompare only when needed
     {
         int      itemsTotal = 0;
         uint64_t bytesTotal = 0;
         for (const BinaryWorkload& bwl : fpWorkload)
         {
-            itemsTotal += bwl.filesToCompareBytewise.size();
+            itemsTotal += static_cast<int>(bwl.filesToCompareBytewise.size());
 
             for (const FilePair* file : bwl.filesToCompareBytewise)
                 bytesTotal += file->getFileSize<SelectSide::left>(); //left and right file sizes are equal
         }
-        cb_.initNewPhase(itemsTotal, bytesTotal, ProcessPhase::comparingContent); //throw X
+        cb_.initNewPhase(itemsTotal, bytesTotal, ProcessPhase::binaryCompare); //throw X
 
         //PERF_START;
 
@@ -705,7 +767,7 @@ const Zstringc* MergeSides::checkFailedRead(FileSystemObject& fsObj, const Zstri
                 it != errorsByRelPath_.end())
                 errorMsg = &it->second;
 
-    if (errorMsg)
+    if (errorMsg) //make sure all items are disabled => avoid user panicking: https://freefilesync.org/forum/viewtopic.php?t=7582
     {
         fsObj.setActive(false);
         fsObj.setCategoryConflict(*errorMsg); //peak memory: Zstringc is ref-counted, unlike std::string!
@@ -739,19 +801,19 @@ void MergeSides::fillOneSide(const FolderContainer& folderCont, const Zstringc*
 {
     forEachSorted(folderCont.files, [&](const Zstring& fileName, const FileAttributes& attrib)
     {
-        FilePair& newItem = output.addSubFile<side>(fileName, attrib);
+        FilePair& newItem = output.addFile<side>(fileName, attrib);
         checkFailedRead(newItem, errorMsg);
     });
 
     forEachSorted(folderCont.symlinks, [&](const Zstring& linkName, const LinkAttributes& attrib)
     {
-        SymlinkPair& newItem = output.addSubLink<side>(linkName, attrib);
+        SymlinkPair& newItem = output.addLink<side>(linkName, attrib);
         checkFailedRead(newItem, errorMsg);
     });
 
     forEachSorted(folderCont.folders, [&](const Zstring& folderName, const std::pair<FolderAttributes, FolderContainer>& attrib)
     {
-        FolderPair& newFolder = output.addSubFolder<side>(folderName, attrib.first);
+        FolderPair& newFolder = output.addFolder<side>(folderName, attrib.first);
         const Zstringc* errorMsgNew = checkFailedRead(newFolder, errorMsg);
 
         fillOneSide<side>(attrib.second, errorMsgNew, newFolder); //recurse
@@ -838,24 +900,24 @@ void MergeSides::mergeTwoSides(const FolderContainer& lhs, const FolderContainer
 
     matchFolders(lhs.files, rhs.files, [&](const FileData& fileLeft, const Zstringc* conflictMsg)
     {
-        FilePair& newItem = output.addSubFile<SelectSide::left >(fileLeft .first, fileLeft .second);
+        FilePair& newItem = output.addFile<SelectSide::left >(fileLeft.first, fileLeft.second);
         checkFailedRead(newItem, conflictMsg ? conflictMsg : errorMsg);
     },
     [&](const FileData& fileRight, const Zstringc* conflictMsg)
     {
-        FilePair& newItem = output.addSubFile<SelectSide::right>(fileRight.first, fileRight.second);
+        FilePair& newItem = output.addFile<SelectSide::right>(fileRight.first, fileRight.second);
         checkFailedRead(newItem, conflictMsg ? conflictMsg : errorMsg);
     },
     [&](const FileData& fileLeft, const FileData& fileRight)
     {
-        FilePair& newItem = output.addSubFile(fileLeft.first,
-                                              fileLeft.second,
-                                              FILE_CONFLICT, //dummy-value until categorization is finished later
-                                              fileRight.first,
-                                              fileRight.second);
+        FilePair& newItem = output.addFile(fileLeft.first,
+                                           fileLeft.second,
+                                           FILE_CONFLICT, //dummy-value until categorization is finished later
+                                           fileRight.first,
+                                           fileRight.second);
         if (!checkFailedRead(newItem, errorMsg))
             undefinedFiles_.push_back(&newItem);
-        static_assert(std::is_same_v<ContainerObject::FileList, std::list<FilePair>>); //ContainerObject::addSubFile() must NOT invalidate references used in "undefinedFiles"!
+        static_assert(std::is_same_v<ContainerObject::FileList, std::list<FilePair>>); //ContainerObject::addFile() must NOT invalidate references used in "undefinedFiles"!
     });
 
     //-----------------------------------------------------------------------------------------------
@@ -863,21 +925,21 @@ void MergeSides::mergeTwoSides(const FolderContainer& lhs, const FolderContainer
 
     matchFolders(lhs.symlinks, rhs.symlinks, [&](const SymlinkData& symlinkLeft, const Zstringc* conflictMsg)
     {
-        SymlinkPair& newItem = output.addSubLink<SelectSide::left >(symlinkLeft .first, symlinkLeft .second);
+        SymlinkPair& newItem = output.addLink<SelectSide::left >(symlinkLeft.first, symlinkLeft.second);
         checkFailedRead(newItem, conflictMsg ? conflictMsg : errorMsg);
     },
     [&](const SymlinkData& symlinkRight, const Zstringc* conflictMsg)
     {
-        SymlinkPair& newItem = output.addSubLink<SelectSide::right>(symlinkRight.first, symlinkRight.second);
+        SymlinkPair& newItem = output.addLink<SelectSide::right>(symlinkRight.first, symlinkRight.second);
         checkFailedRead(newItem, conflictMsg ? conflictMsg : errorMsg);
     },
     [&](const SymlinkData& symlinkLeft, const SymlinkData& symlinkRight) //both sides
     {
-        SymlinkPair& newItem = output.addSubLink(symlinkLeft.first,
-                                                 symlinkLeft.second,
-                                                 SYMLINK_CONFLICT, //dummy-value until categorization is finished later
-                                                 symlinkRight.first,
-                                                 symlinkRight.second);
+        SymlinkPair& newItem = output.addLink(symlinkLeft.first,
+                                              symlinkLeft.second,
+                                              SYMLINK_CONFLICT, //dummy-value until categorization is finished later
+                                              symlinkRight.first,
+                                              symlinkRight.second);
         if (!checkFailedRead(newItem, errorMsg))
             undefinedSymlinks_.push_back(&newItem);
     });
@@ -887,19 +949,19 @@ void MergeSides::mergeTwoSides(const FolderContainer& lhs, const FolderContainer
 
     matchFolders(lhs.folders, rhs.folders, [&](const FolderData& dirLeft, const Zstringc* conflictMsg)
     {
-        FolderPair& newFolder = output.addSubFolder<SelectSide::left>(dirLeft.first, dirLeft.second.first);
+        FolderPair& newFolder = output.addFolder<SelectSide::left>(dirLeft.first, dirLeft.second.first);
         const Zstringc* errorMsgNew = checkFailedRead(newFolder, conflictMsg ? conflictMsg : errorMsg);
         this->fillOneSide<SelectSide::left>(dirLeft.second.second, errorMsgNew, newFolder); //recurse
     },
     [&](const FolderData& dirRight, const Zstringc* conflictMsg)
     {
-        FolderPair& newFolder = output.addSubFolder<SelectSide::right>(dirRight.first, dirRight.second.first);
+        FolderPair& newFolder = output.addFolder<SelectSide::right>(dirRight.first, dirRight.second.first);
         const Zstringc* errorMsgNew = checkFailedRead(newFolder, conflictMsg ? conflictMsg : errorMsg);
         this->fillOneSide<SelectSide::right>(dirRight.second.second, errorMsgNew, newFolder); //recurse
     },
     [&](const FolderData& dirLeft, const FolderData& dirRight)
     {
-        FolderPair& newFolder = output.addSubFolder(dirLeft.first, dirLeft.second.first, DIR_EQUAL, dirRight.first, dirRight.second.first);
+        FolderPair& newFolder = output.addFolder(dirLeft.first, dirLeft.second.first, DIR_EQUAL, dirRight.first, dirRight.second.first);
         const Zstringc* errorMsgNew = checkFailedRead(newFolder, errorMsg);
 
         if (!errorMsgNew)
@@ -940,39 +1002,65 @@ void stripExcludedDirectories(ContainerObject& hierObj, const PathFilter& filter
 
 
 //create comparison result table and fill category except for files existing on both sides: undefinedFiles and undefinedSymlinks are appended!
-std::shared_ptr<BaseFolderPair> ComparisonBuffer::performComparison(const ResolvedFolderPair& fp,
-                                                                    const FolderPairCfg& fpCfg,
-                                                                    std::vector<FilePair*>& undefinedFiles,
-                                                                    std::vector<SymlinkPair*>& undefinedSymlinks) const
+SharedRef<BaseFolderPair> ComparisonBuffer::performComparison(const ResolvedFolderPair& fp,
+                                                              const FolderPairCfg& fpCfg,
+                                                              std::vector<FilePair*>& undefinedFiles,
+                                                              std::vector<SymlinkPair*>& undefinedSymlinks) const
 {
     cb_.updateStatus(_("Generating file list...")); //throw X
     cb_.requestUiUpdate(true /*force*/); //throw X
 
+    const BaseFolderStatus folderStatusL = getBaseFolderStatus(fp.folderPathLeft);
+    const BaseFolderStatus folderStatusR = getBaseFolderStatus(fp.folderPathRight);
+
+
     std::unordered_map<ZstringNoCase, Zstringc> failedReads; //base-relative paths or empty if read-error for whole base directory
+    const FolderContainer* folderContL = nullptr;
+    const FolderContainer* folderContR = nullptr;
 
-    auto evalFolderContent = [&](const AbstractPath& folderPath) -> const FolderContainer&
+
+    const FolderContainer empty;
+    if (folderStatusL == BaseFolderStatus::failure ||
+        folderStatusR == BaseFolderStatus::failure)
     {
-        const DirectoryValue& dirVal = folderBuffer_.find({folderPath, fpCfg.filter.nameFilter, fpCfg.handleSymlinks})->second;
-        //contract: folderBuffer_ has entries for *all* folders (existing or not)
+        auto it = folderStatus_.failedChecks.find(fp.folderPathLeft);
+        if (it == folderStatus_.failedChecks.end())
+            it = folderStatus_.failedChecks.find(fp.folderPathRight);
 
-        //mix failedFolderReads with failedItemReads:
-        //associate folder traversing errors with folder (instead of child items only) to show on GUI! See "MergeSides"
-        //=> minor pessimization for "excludefilterFailedRead" which needlessly excludes parent folders, too
-        auto append = [&](const std::unordered_map<Zstring, Zstringc>& c)
+        failedReads[Zstring() /*empty string for root*/] = utfTo<Zstringc>(it->second.toString());
+
+        folderContL = &empty; //no need to list or display one-sided results if
+        folderContR = &empty; //*any* folder existence check fails (even if other side would have been in folderBuffer_!)
+    }
+    else
+    {
+        auto evalBuffer = [&](const AbstractPath& folderPath, const FolderContainer*& folderCont)
         {
-            for (const auto& [relPath, errorMsg] : c)
-                failedReads.emplace(relPath, errorMsg);
+            auto it = folderBuffer_.find({folderPath, fpCfg.filter.nameFilter, fpCfg.handleSymlinks});
+            if (it != folderBuffer_.end())
+            {
+                const DirectoryValue& dirVal = it->second;
+
+                //mix failedFolderReads with failedItemReads:
+                //associate folder traversing errors with folder (instead of child items only) to show on GUI! See "MergeSides"
+                //=> minor pessimization for "excludefilterFailedRead" which needlessly excludes parent folders, too
+                failedReads.insert(dirVal.failedFolderReads.begin(), dirVal.failedFolderReads.end());
+                failedReads.insert(dirVal.failedItemReads  .begin(), dirVal.failedItemReads  .end());
+
+                assert(getBaseFolderStatus(folderPath) == BaseFolderStatus::existing);
+                folderCont = &dirVal.folderCont;
+            }
+            else
+            {
+                assert(getBaseFolderStatus(folderPath) == BaseFolderStatus::notExisting); //including AFS::isNullPath()
+                folderCont = &empty;
+            }
         };
-        append(dirVal.failedFolderReads);
-        append(dirVal.failedItemReads);
+        evalBuffer(fp.folderPathLeft,  folderContL);
+        evalBuffer(fp.folderPathRight, folderContR);
+    }
 
-        return dirVal.folderCont;
-    };
 
-    const FolderContainer& folderContL = evalFolderContent(fp.folderPathLeft);
-    const FolderContainer& folderContR = evalFolderContent(fp.folderPathRight);
-
-    //*after* evalFolderContent():
     Zstring excludefilterFailedRead;
     if (failedReads.contains(Zstring())) //empty path if read-error for whole base directory
         excludefilterFailedRead += Zstr("*\n");
@@ -986,28 +1074,16 @@ std::shared_ptr<BaseFolderPair> ComparisonBuffer::performComparison(const Resolv
     if constexpr (FILE_NAME_SEPARATOR != Zstr('\\')) replace(excludefilterFailedRead, Zstr('\\'), Zstr('?'));
 
 
-    auto getBaseFolderStatus = [&](const AbstractPath& folderPath)
-    {
-        if (folderStatus_.existing.contains(folderPath))
-            return BaseFolderStatus::existing;
-        if (folderStatus_.notExisting.contains(folderPath))
-            return BaseFolderStatus::notExisting;
-        if (folderStatus_.failedChecks.contains(folderPath))
-            return BaseFolderStatus::failure;
-        assert(AFS::isNullPath(folderPath));
-        return BaseFolderStatus::notExisting;
-    };
-
-    std::shared_ptr<BaseFolderPair> output = std::make_shared<BaseFolderPair>(fp.folderPathLeft,
-                                                                              getBaseFolderStatus(fp.folderPathLeft), //dir existence must be checked only once!
-                                                                              fp.folderPathRight,
-                                                                              getBaseFolderStatus(fp.folderPathRight),
-                                                                              fpCfg.filter.nameFilter.ref().copyFilterAddingExclusion(excludefilterFailedRead),
-                                                                              fpCfg.compareVar,
-                                                                              fileTimeTolerance_,
-                                                                              fpCfg.ignoreTimeShiftMinutes);
+    SharedRef<BaseFolderPair> output = makeSharedRef<BaseFolderPair>(fp.folderPathLeft,
+                                                                     folderStatusL, //check folder existence only once!
+                                                                     fp.folderPathRight,
+                                                                     folderStatusR, //
+                                                                     fpCfg.filter.nameFilter.ref().copyFilterAddingExclusion(excludefilterFailedRead),
+                                                                     fpCfg.compareVar,
+                                                                     fileTimeTolerance_,
+                                                                     fpCfg.ignoreTimeShiftMinutes);
     //PERF_START;
-    MergeSides(failedReads, undefinedFiles, undefinedSymlinks).execute(folderContL, folderContR, *output);
+    MergeSides(failedReads, undefinedFiles, undefinedSymlinks).execute(*folderContL, *folderContR, output.ref());
     //PERF_STOP;
 
     //##################### in/exclude rows according to filtering #####################
@@ -1015,10 +1091,10 @@ std::shared_ptr<BaseFolderPair> ComparisonBuffer::performComparison(const Resolv
 
     //attention: some excluded directories are still in the comparison result! (see include filter handling!)
     if (!fpCfg.filter.nameFilter.ref().isNull())
-        stripExcludedDirectories(*output, fpCfg.filter.nameFilter.ref()); //mark excluded directories (see parallelDeviceTraversal()) + remove superfluous excluded subdirectories
+        stripExcludedDirectories(output.ref(), fpCfg.filter.nameFilter.ref()); //mark excluded directories (see parallelDeviceTraversal()) + remove superfluous excluded subdirectories
 
     //apply soft filtering (hard filter already applied during traversal!)
-    addSoftFiltering(*output, fpCfg.filter.timeSizeFilter);
+    addSoftFiltering(output.ref(), fpCfg.filter.timeSizeFilter);
 
     //##################################################################################
     return output;
@@ -1035,11 +1111,9 @@ FolderComparison fff::compare(WarningDialogs& warnings,
                               const std::vector<FolderPairCfg>& fpCfgList,
                               ProcessCallback& callback /*throw X*/) //throw X
 {
-    //PERF_START;
-
     //indicator at the very beginning of the log to make sense of "total time"
     //init process: keep at beginning so that all gui elements are initialized properly
-    callback.initNewPhase(-1, -1, ProcessPhase::scanning); //throw X; it's unknown how many files will be scanned => -1 objects
+    callback.initNewPhase(-1, -1, ProcessPhase::scan); //throw X; it's unknown how many files will be scanned => -1 objects
     //callback.logInfo(Comparison started")); -> still useful?
 
     //-------------------------------------------------------------------------------
@@ -1067,7 +1141,7 @@ FolderComparison fff::compare(WarningDialogs& warnings,
                                                                requestPassword, warnings, callback); //throw X
     //directory existence only checked *once* to avoid race conditions!
     if (resInfo.resolvedPairs.size() != fpCfgList.size())
-        throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
+        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
 
     std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>> workLoad;
     for (size_t i = 0; i < fpCfgList.size(); ++i)
@@ -1137,51 +1211,18 @@ FolderComparison fff::compare(WarningDialogs& warnings,
         //reduce peak memory by restricting lifetime of ComparisonBuffer to have ended when loading potentially huge InSyncFolder instance in redetermineSyncDirection()
         {
             //------------------- fill directory buffer: traverse/read folders --------------------------
-            std::set<DirectoryKey> folderKeys;
-            for (const auto& [folderPair, fpCfg] : workLoad)
-            {
-                folderKeys.emplace(DirectoryKey({folderPair.folderPathLeft,  fpCfg.filter.nameFilter, fpCfg.handleSymlinks}));
-                folderKeys.emplace(DirectoryKey({folderPair.folderPathRight, fpCfg.filter.nameFilter, fpCfg.handleSymlinks}));
-            }
-
+            ComparisonBuffer cmpBuf(resInfo.baseFolderStatus,
+                                    fileTimeTolerance, callback);
             //PERF_START;
-            ComparisonBuffer cmpBuff(folderKeys,
-                                     resInfo.baseFolderStatus,
-                                     fileTimeTolerance, callback);
+            output = cmpBuf.execute(workLoad);
             //PERF_STOP;
-
-            //process binary comparison as one junk
-            std::vector<std::pair<ResolvedFolderPair, FolderPairCfg>> workLoadByContent;
-            for (const auto& [folderPair, fpCfg] : workLoad)
-                if (fpCfg.compareVar == CompareVariant::content)
-                    workLoadByContent.push_back({folderPair, fpCfg});
-
-            std::vector<std::shared_ptr<BaseFolderPair>> outputByContent = cmpBuff.compareByContent(workLoadByContent);
-            auto itOByC = outputByContent.begin();
-
-            //write output in expected order
-            for (const auto& [folderPair, fpCfg] : workLoad)
-                switch (fpCfg.compareVar)
-                {
-                    case CompareVariant::timeSize:
-                        output.push_back(cmpBuff.compareByTimeSize(folderPair, fpCfg));
-                        break;
-                    case CompareVariant::size:
-                        output.push_back(cmpBuff.compareBySize(folderPair, fpCfg));
-                        break;
-                    case CompareVariant::content:
-                        assert(itOByC != outputByContent.end());
-                        if (itOByC != outputByContent.end())
-                            output.push_back(*itOByC++);
-                        break;
-                }
         }
         assert(output.size() == fpCfgList.size());
 
         //--------- set initial sync-direction --------------------------------------------------
         std::vector<std::pair<BaseFolderPair*, SyncDirectionConfig>> directCfgs;
         for (auto it = output.begin(); it != output.end(); ++it)
-            directCfgs.emplace_back(&** it, fpCfgList[it - output.begin()].directionCfg);
+            directCfgs.emplace_back(&it->ref(), fpCfgList[it - output.begin()].directionCfg);
 
         redetermineSyncDirection(directCfgs,
                                  callback); //throw X
