@@ -7,7 +7,7 @@
 #include "batch_config.h"
 #include <wx/wupdlock.h>
 //#include <wx+/std_button_layout.h>
-#include <wx+/font_size.h>
+#include <wx+/window_layout.h>
 #include <wx+/image_resources.h>
 #include <wx+/image_tools.h>
 #include <wx+/choice_enum.h>
@@ -56,7 +56,7 @@ private:
     //output-only parameters
     BatchDialogConfig& dlgCfgOut_;
 
-    EnumDescrList<PostSyncAction> enumPostSyncAction_;
+    EnumDescrList<PostBatchAction> enumPostBatchAction_;
 };
 
 //###################################################################################################################################
@@ -72,17 +72,20 @@ BatchDialog::BatchDialog(wxWindow* parent, BatchDialogConfig& dlgCfg) :
 
     setImage(*m_bitmapBatchJob, loadImage("cfg_batch"));
 
-    enumPostSyncAction_.
-    add(PostSyncAction::none,     L"").
-    add(PostSyncAction::sleep,    _("System: Sleep")).
-    add(PostSyncAction::shutdown, _("System: Shut down"));
+    enumPostBatchAction_.
+    add(PostBatchAction::none,     L"").
+    add(PostBatchAction::sleep,    _("System: Sleep")).
+    add(PostBatchAction::shutdown, _("System: Shut down"));
 
     setConfig(dlgCfg);
 
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); }); //enable dialog-specific key events
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonSaveAs->SetFocus();
@@ -122,7 +125,7 @@ void BatchDialog::setConfig(const BatchDialogConfig& dlgCfg)
 
     m_checkBoxRunMinimized->SetValue(dlgCfg.batchExCfg.runMinimized);
     m_checkBoxAutoClose   ->SetValue(dlgCfg.batchExCfg.autoCloseSummary);
-    setEnumVal(enumPostSyncAction_, *m_choicePostSyncAction, dlgCfg.batchExCfg.postSyncAction);
+    setEnumVal(enumPostBatchAction_, *m_choicePostSyncAction, dlgCfg.batchExCfg.postBatchAction);
 
     updateGui(); //re-evaluate gui after config changes
 }
@@ -134,10 +137,10 @@ BatchDialogConfig BatchDialog::getConfig() const
     {
         .batchExCfg
         {
-            .batchErrorHandling  = m_radioBtnErrorDialogCancel->GetValue() ? BatchErrorHandling::cancel : BatchErrorHandling::showPopup,
             .runMinimized        = m_checkBoxRunMinimized->GetValue(),
             .autoCloseSummary    = m_checkBoxAutoClose   ->GetValue(),
-            .postSyncAction = getEnumVal(enumPostSyncAction_, *m_choicePostSyncAction),
+            .batchErrorHandling  = m_radioBtnErrorDialogCancel->GetValue() ? BatchErrorHandling::cancel : BatchErrorHandling::showPopup,
+            .postBatchAction = getEnumVal(enumPostBatchAction_, *m_choicePostSyncAction),
         },
         .ignoreErrors = m_checkBoxIgnoreErrors->GetValue(),
     };
