@@ -22,6 +22,7 @@
 #include "../status_handler.h"
 #include "../base/algorithm.h"
 #include "../return_codes.h"
+#include "../base/synchronization.h"
 
 
 namespace fff
@@ -88,6 +89,7 @@ private:
     void setViewFilterDefault();
 
     void cfgHistoryRemoveObsolete(const std::vector<Zstring>& filepaths);
+    void cfgHistoryUpdateNotes   (const std::vector<Zstring>& filepaths);
 
     void insertAddFolderPair(const std::vector<LocalPairConfig>& newPairs, size_t pos);
     void moveAddFolderPairUp(size_t pos);
@@ -102,7 +104,7 @@ private:
     void updateGuiDelayedIf(bool condition); //400 ms delay
 
     void updateGridViewData();     //
-    void updateStatistics();       // more fine-grained updaters
+    void updateStatistics(const SyncStatistics& st); // more fine-grained updaters
     void updateUnsavedCfgStatus(); //
 
     std::vector<std::wstring> getJobNames() const;
@@ -123,6 +125,9 @@ private:
     void deleteSelectedFiles(const std::vector<FileSystemObject*>& selectionL,
                              const std::vector<FileSystemObject*>& selectionR, bool moveToRecycler);
 
+    void renameSelectedFiles(const std::vector<FileSystemObject*>& selectionL,
+                             const std::vector<FileSystemObject*>& selectionR);
+
     void openExternalApplication(const Zstring& commandLinePhrase, bool leftSide,
                                  const std::vector<FileSystemObject*>& selectionL,
                                  const std::vector<FileSystemObject*>& selectionR); //selection may be empty
@@ -229,8 +234,6 @@ private:
 
     void showConfigDialog(SyncConfigPanel panelToShow, int localPairIndexToShow);
 
-    void updateConfigLastRunStats(time_t lastRunTime, SyncResult result, const AbstractPath& logFilePath);
-
     void setLastOperationLog(const ProcessSummary& summary, const std::shared_ptr<const zen::ErrorLog>& errorLog);
     void showLogPanel(bool show);
 
@@ -303,12 +306,20 @@ private:
 
     XmlGuiConfig lastSavedCfg_; //support for: "Save changed configuration?" dialog
 
-    const Zstring lastRunConfigPath_ = getLastRunConfigPath(); //let's not use another static...
+    const Zstring lastRunConfigPath_ = getLastRunConfigPath(); //let's not use another global...
     //-------------------------------------
 
     //the prime data structure of this tool *bling*:
     FolderComparison folderCmp_; //optional!: sync button not available if empty
-    std::shared_ptr<const zen::ErrorLog> errorLogCmp_;
+
+    //merge logs of individual steps (comparison, manual operations, sync) into a combined result (just as for ffs_batch jobs)
+    struct FullSyncLog
+    {
+        zen::ErrorLog log;
+        std::chrono::system_clock::time_point startTime;
+        std::chrono::milliseconds totalTime{};
+    };
+    std::optional<FullSyncLog> fullSyncLog_;
 
     //folder pairs:
     std::unique_ptr<FolderPairFirst> firstFolderPair_; //always bound!!!
@@ -328,7 +339,7 @@ private:
     std::vector<wxString> statusTxts_; //the first one is the original/non-flash status message
     bool statusTxtHighlightFirst_ = false;
 
-    //compare status panel (hidden on start, shown when comparing)
+    //compare status panel (hidden on start, shown during comparison)
     std::unique_ptr<CompareProgressPanel> compareStatus_; //always bound
 
     LogPanel* logPanel_ = nullptr;
@@ -345,7 +356,7 @@ private:
     time_t manualTimeSpanFrom_ = 0;
     time_t manualTimeSpanTo_   = 0; //buffer manual time span selection at session level
 
-    //regenerate view filter button labels only when necessary:
+    //recreate view filter button labels only when necessary:
     std::unordered_map<const zen::ToggleButton*, int /*itemCount*/> buttonLabelItemCount_;
 
     const std::shared_ptr<HistoryList> folderHistoryLeft_;  //shared by all wxComboBox dropdown controls
