@@ -20,7 +20,7 @@
 #include <wx+/rtl.h>
 #include <wx+/no_flicker.h>
 #include <wx+/image_tools.h>
-#include <wx+/font_size.h>
+#include <wx+/window_layout.h>
 #include <wx+/popup_dlg.h>
 #include <wx+/async_task.h>
 #include <wx+/image_resources.h>
@@ -58,7 +58,7 @@ private:
     void onOkay  (wxCommandEvent& event) override { EndModal(static_cast<int>(ConfirmationButton::accept)); }
     void onClose (wxCloseEvent&   event) override { EndModal(static_cast<int>(ConfirmationButton::cancel)); }
     void onOpenForum(wxCommandEvent& event) override { wxLaunchDefaultBrowser(L"https://freefilesync.org/forum"); }
-    void onSendEmail(wxCommandEvent& event) override { wxLaunchDefaultBrowser(L"mailto:zenju@" L"freefilesync.org"); }
+    void onSendEmail(wxCommandEvent& event) override { wxLaunchDefaultBrowser(wxString() + L"mailto:zenju@" + /*don't leave full email in either source or binary*/ L"freefilesync.org"); }
     void onDonate   (wxCommandEvent& event) override { wxLaunchDefaultBrowser(L"https://freefilesync.org/donate"); }
 
     void onLocalKeyEvent(wxKeyEvent& event);
@@ -75,8 +75,8 @@ AboutDlg::AboutDlg(wxWindow* parent) : AboutDlgGenerated(parent)
     setImage(*m_bitmapLogoLeft, loadImage("logo-left"));
 
     setBitmapTextLabel(*m_bpButtonForum, loadImage("ffs_forum"), L"FreeFileSync Forum");
-    setBitmapTextLabel(*m_bpButtonEmail, loadImage("ffs_email"), L"zenju@" L"freefilesync.org");
-    m_bpButtonEmail->SetToolTip(L"mailto:zenju@" L"freefilesync.org");
+    setBitmapTextLabel(*m_bpButtonEmail, loadImage("ffs_email"), wxString() + L"zenju@" + /*don't leave full email in either source or binary*/ L"freefilesync.org");
+    m_bpButtonEmail->SetToolTip(                          wxString() + L"mailto:zenju@" + /*don't leave full email in either source or binary*/ L"freefilesync.org");
 
 
     wxString build = utfTo<wxString>(ffsVersion);
@@ -144,6 +144,10 @@ AboutDlg::AboutDlg(wxWindow* parent) : AboutDlgGenerated(parent)
     //--------------------------------------------------------------------------
     //have animal + text match *final* dialog width
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
 
     {
         const int imageWidth = (m_panelDonate->GetSize().GetWidth() - 5 - 5 - 5 /* grey border*/) / 2;
@@ -159,7 +163,10 @@ AboutDlg::AboutDlg(wxWindow* parent) : AboutDlgGenerated(parent)
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); }); //enable dialog-specific key events
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonClose->SetFocus(); //on GTK ESC is only associated with wxID_OK correctly if we set at least *any* focus at all!!!
@@ -278,10 +285,10 @@ CloudSetupDlg::CloudSetupDlg(wxWindow* parent, Zstring& folderPathPhrase, Zstrin
     m_textCtrlServer->SetHint(_("Example:") + L"    website.com    66.198.240.22");
     m_textCtrlServer->SetMinSize({fastFromDIP(260), -1});
 
-    m_textCtrlPort            ->SetMinSize({fastFromDIP(60), -1}); //
-    m_spinCtrlConnectionCount ->SetMinSize({fastFromDIP(70), -1}); //Hack: set size (why does wxWindow::Size() not work?)
-    m_spinCtrlChannelCountSftp->SetMinSize({fastFromDIP(70), -1}); //
-    m_spinCtrlTimeout         ->SetMinSize({fastFromDIP(70), -1}); //
+    m_textCtrlPort->SetMinSize({fastFromDIP(60), -1});
+    setDefaultWidth(*m_spinCtrlConnectionCount);
+    setDefaultWidth(*m_spinCtrlChannelCountSftp);
+    setDefaultWidth(*m_spinCtrlTimeout);
 
     setupFileDrop(*m_panelAuth);
     m_panelAuth->Bind(EVENT_DROP_FILE, [this](FileDropEvent& event) { onKeyFileDropped(event); });
@@ -393,7 +400,11 @@ CloudSetupDlg::CloudSetupDlg(wxWindow* parent, Zstring& folderPathPhrase, Zstrin
     m_checkBoxPasswordPrompt->Hide();
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+    //=> works like a charm for GTK with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     updateGui(); //*after* SetSizeHints when standard dialog height has been calculated
@@ -527,17 +538,17 @@ void CloudSetupDlg::onDetectServerChannelLimit(wxCommandEvent& event)
         m_spinCtrlChannelCountSftp->SetSelection(0, 0); //some visual feedback: clear selection
         m_spinCtrlChannelCountSftp->Refresh(); //both needed for wxGTK: meh!
         m_spinCtrlChannelCountSftp->Update();  //
-        
+
         AbstractPath folderPath = getFolderPath(); //noexcept
         //-------------------------------------------------------------------
         auto requestPassword = [&, password = Zstring()](const std::wstring& msg, const std::wstring& lastErrorMsg) mutable
         {
             assert(runningOnMainThread());
             if (showPasswordPrompt(this, msg, lastErrorMsg, password) != ConfirmationButton::accept)
-                throw AbortProcess();
+                throw CancelProcess();
             return password;
         };
-        AFS::authenticateAccess(folderPath.afsDevice, requestPassword); //throw FileError, AbortProcess
+        AFS::authenticateAccess(folderPath.afsDevice, requestPassword); //throw FileError, CancelProcess
         //-------------------------------------------------------------------
 
         const int channelCountMax = getServerMaxChannelsPerConnection(extractSftpLogin(folderPath.afsDevice)); //throw FileError
@@ -546,7 +557,7 @@ void CloudSetupDlg::onDetectServerChannelLimit(wxCommandEvent& event)
         m_spinCtrlChannelCountSftp->SetFocus(); //[!] otherwise selection is lost
         m_spinCtrlChannelCountSftp->SetSelection(-1, -1); //some visual feedback: select all
     }
-        catch (AbortProcess&) { return; }
+    catch (CancelProcess&) { return; }
     catch (const FileError& e)
     {
         showNotificationDialog(this, DialogInfoType::error, PopupDialogCfg().setDetailInstructions(e.toString()));
@@ -575,7 +586,7 @@ void CloudSetupDlg::onTypingPassword(wxCommandEvent& event)
     assert(m_staticTextPassword->IsShown());
     const wxString password = (m_checkBoxShowPassword->GetValue() ? m_textCtrlPasswordVisible : m_textCtrlPasswordHidden)->GetValue();
     if (m_checkBoxShowPassword  ->IsShown() != !password.empty() || //let's avoid some minor flicker
-        m_checkBoxPasswordPrompt->IsShown() !=  password.empty())   //in updateGui() Layout()
+        m_checkBoxPasswordPrompt->IsShown() !=  password.empty())   //in updateGui() Dimensions()
         updateGui();
 }
 
@@ -823,18 +834,18 @@ void CloudSetupDlg::onBrowseCloudFolder(wxCommandEvent& event)
     AbstractPath folderPath = getFolderPath(); //noexcept
     try
     {
-                //-------------------------------------------------------------------
+        //-------------------------------------------------------------------
         auto requestPassword = [&, password = Zstring()](const std::wstring& msg, const std::wstring& lastErrorMsg) mutable
         {
             assert(runningOnMainThread());
             if (showPasswordPrompt(this, msg, lastErrorMsg, password) != ConfirmationButton::accept)
-                throw AbortProcess();
+                throw CancelProcess();
             return password;
         };
-        AFS::authenticateAccess(folderPath.afsDevice, requestPassword); //throw FileError, AbortProcess
+        AFS::authenticateAccess(folderPath.afsDevice, requestPassword); //throw FileError, CancelProcess
         //caveat: this could block *indefinitely* for Google Drive, but luckily already authenticated in this context
         //-------------------------------------------------------------------
-        //         
+        //
         //for (S)FTP it makes more sense to start with the home directory rather than root (which often denies access!)
         if (!AFS::getParentPath(folderPath))
         {
@@ -845,7 +856,7 @@ void CloudSetupDlg::onBrowseCloudFolder(wxCommandEvent& event)
                 folderPath.afsPath = getFtpHomePath(extractFtpLogin(folderPath.afsDevice)); //throw FileError
         }
     }
-    catch (AbortProcess&) { return; }
+    catch (CancelProcess&) { return; }
     catch (const FileError& e)
     {
         showNotificationDialog(this, DialogInfoType::error, PopupDialogCfg().setDetailInstructions(e.toString()));
@@ -961,7 +972,10 @@ CopyToDialog::CopyToDialog(wxWindow* parent,
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); }); //enable dialog-specific key events
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonOK->SetFocus();
@@ -1076,7 +1090,10 @@ DeleteDialog::DeleteDialog(wxWindow* parent,
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); }); //enable dialog-specific key events
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonOK->SetFocus();
@@ -1216,7 +1233,10 @@ SyncConfirmationDlg::SyncConfirmationDlg(wxWindow* parent,
     setIntValue(*m_staticTextDeleteRight, st.deleteCount<SelectSide::right>(), *m_bitmapDeleteRight, "so_delete_right_sicon");
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonStartSync->SetFocus();
@@ -1380,16 +1400,16 @@ OptionsDlg::OptionsDlg(wxWindow* parent, XmlGlobalSettings& globalCfg) :
 
     logFolderSelector_.setPath(globalCfg.logFolderPhrase);
 
-    m_spinCtrlLogFilesMaxAge->SetMinSize({fastFromDIP(70), -1}); //Hack: set size (why does wxWindow::Size() not work?)
+    setDefaultWidth(*m_spinCtrlLogFilesMaxAge);
 
     setImage(*m_bitmapSettings,           loadImage("settings"));
     setImage(*m_bitmapWarnings,           loadImage("msg_warning", fastFromDIP(20)));
     setImage(*m_bitmapLogFile,            loadImage("log_file",    fastFromDIP(20)));
     setImage(*m_bitmapNotificationSounds, loadImage("notification_sounds"));
     setImage(*m_bitmapConsole,            loadImage("command_line", fastFromDIP(20)));
-    setImage(*m_bitmapCompareDone,        loadImage("compare_sicon"));
-    setImage(*m_bitmapSyncDone,           loadImage("start_sync_sicon"));
-    setImage(*m_bitmapAlertPending,       loadImage("msg_error", loadImage("compare_sicon").GetSize().x));
+    setImage(*m_bitmapCompareDone,        loadImage("compare", fastFromDIP(20)));
+    setImage(*m_bitmapSyncDone,           loadImage("start_sync", fastFromDIP(20)));
+    setImage(*m_bitmapAlertPending,       loadImage("msg_error", fastFromDIP(20)));
     setImage(*m_bpButtonPlayCompareDone,  loadImage("play_sound"));
     setImage(*m_bpButtonPlaySyncDone,     loadImage("play_sound"));
     setImage(*m_bpButtonPlayAlertPending, loadImage("play_sound"));
@@ -1400,7 +1420,7 @@ OptionsDlg::OptionsDlg(wxWindow* parent, XmlGlobalSettings& globalCfg) :
     m_checkListHiddenDialogs->Hide();
     m_buttonShowCtxCustomize->Hide();
 
-    //fix wxCheckListBox's stupid "per-item toggle"
+    //fix wxCheckListBox's stupid "per-item toggle" when multiple items are selected
     m_checkListHiddenDialogs->Bind(wxEVT_KEY_DOWN, [&checklist = *m_checkListHiddenDialogs](wxKeyEvent& event)
     {
         switch (event.GetKeyCode())
@@ -1480,7 +1500,10 @@ OptionsDlg::OptionsDlg(wxWindow* parent, XmlGlobalSettings& globalCfg) :
     updateGui();
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     //restore actual value:
@@ -1528,7 +1551,7 @@ void OptionsDlg::updateGui()
 
 void OptionsDlg::expandConfigArea(ConfigArea area)
 {
-    //only show one expaned area at a time (wxGTK even crashes when showing both: not worth debugging)
+    //only show one expanded area at a time (wxGTK even crashes when showing both: not worth debugging)
     m_buttonShowHiddenDialogs->Show(area != ConfigArea::hidden);
     m_buttonShowCtxCustomize ->Show(area != ConfigArea::context);
 
@@ -1800,7 +1823,10 @@ SelectTimespanDlg::SelectTimespanDlg(wxWindow* parent, time_t& timeFrom, time_t&
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); }); //enable dialog-specific key events
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonOkay->SetFocus();
@@ -1896,7 +1922,10 @@ PasswordPromptDlg::PasswordPromptDlg(wxWindow* parent, const std::wstring& msg,
     m_textCtrlPasswordVisible->Hide();
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     updateGui(); //*after* SetSizeHints when standard dialog height has been calculated
@@ -1974,12 +2003,15 @@ CfgHighlightDlg::CfgHighlightDlg(wxWindow* parent, int& cfgHistSyncOverdueDays)
 
     m_staticTextHighlight->Wrap(fastFromDIP(300));
 
-    m_spinCtrlOverdueDays->SetMinSize({fastFromDIP(70), -1}); //Hack: set size (why does wxWindow::Size() not work?)
+    setDefaultWidth(*m_spinCtrlOverdueDays);
 
     m_spinCtrlOverdueDays->SetValue(cfgHistSyncOverdueDays);
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_spinCtrlOverdueDays->SetFocus();
@@ -2045,7 +2077,10 @@ ActivationDlg::ActivationDlg(wxWindow* parent,
     m_textCtrlOfflineActivationKey->ChangeValue(manualActivationKey);
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     m_buttonActivateOnline->SetFocus();
@@ -2063,14 +2098,14 @@ void ActivationDlg::onCopyUrl(wxCommandEvent& event)
 
 void ActivationDlg::onActivateOnline(wxCommandEvent& event)
 {
-    manualActivationKeyOut_ = m_textCtrlOfflineActivationKey->GetValue();
+    manualActivationKeyOut_ = utfTo<std::wstring>(m_textCtrlOfflineActivationKey->GetValue());
     EndModal(static_cast<int>(ActivationDlgButton::activateOnline));
 }
 
 
 void ActivationDlg::onActivateOffline(wxCommandEvent& event)
 {
-    manualActivationKeyOut_ = m_textCtrlOfflineActivationKey->GetValue();
+    manualActivationKeyOut_ = utfTo<std::wstring>(m_textCtrlOfflineActivationKey->GetValue());
     if (trimCpy(manualActivationKeyOut_).empty()) //alternative: disable button? => user thinks option is not available!
     {
         showNotificationDialog(this, DialogInfoType::info, PopupDialogCfg().setMainInstructions(_("Please enter a key for offline activation.")));
@@ -2150,8 +2185,12 @@ DownloadProgressWindow::Impl::Impl(wxWindow* parent, int64_t fileSizeTotal) :
     updateGui();
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
+
     Show();
 
     //clear gui flicker: window must be visible to make this work!
