@@ -14,7 +14,7 @@
 #include <wx+/context_menu.h>
 #include <wx+/choice_enum.h>
 #include <wx+/image_tools.h>
-#include <wx+/font_size.h>
+#include <wx+/window_layout.h>
 #include <wx+/popup_dlg.h>
 #include <wx+/image_resources.h>
 #include <wx+/window_tools.h>
@@ -78,11 +78,9 @@ void initBitmapRadioButtons(const std::vector<std::pair<ToggleButton*, std::stri
 
     for (auto& [btn, imgName] : buttons)
     {
-        wxImage imgSelBack = renderSelectedButton(maxExtent).ConvertToImage();
-        convertToVanillaImage(imgSelBack);
-
-        btn->init(layOver(imgSelBack, generateSelectImage(*btn, imgName, true /*selected*/), wxALIGN_CENTER_VERTICAL | (physicalLeft ? wxALIGN_LEFT : wxALIGN_RIGHT)),
-                  resizeCanvas(labelsNotSel[btn], maxExtent,                                 wxALIGN_CENTER_VERTICAL | (physicalLeft ? wxALIGN_LEFT : wxALIGN_RIGHT)));
+        btn->init(layOver(rectangleImage(maxExtent, getColorToggleButtonFill(), getColorToggleButtonBorder(), fastFromDIP(1)),
+                          generateSelectImage(*btn, imgName, true /*selected*/), wxALIGN_CENTER_VERTICAL | (physicalLeft ? wxALIGN_LEFT : wxALIGN_RIGHT)),
+                  resizeCanvas(labelsNotSel[btn], maxExtent,                     wxALIGN_CENTER_VERTICAL | (physicalLeft ? wxALIGN_LEFT : wxALIGN_RIGHT)));
 
         btn->SetMinSize(maxExtent); //get rid of selection border on Windows :)
         //SetMinSize() instead of SetSize() is needed here for wxWindows layout determination to work correctly
@@ -127,73 +125,76 @@ bool sanitizeFilter(FilterConfig& filterCfg, const std::vector<AbstractPath>& ba
         removeDuplicates(folderPathsPf);
     }
 
-    if (!folderPathsPf.empty())
+
+    std::vector<std::pair<Zstring /*from*/, Zstring /*to*/>> replacements;
+
+    auto replaceFullPaths = [&](Zstring& filterPhrase)
     {
-        std::vector<std::pair<Zstring /*from*/, Zstring /*to*/>> replacements;
+        Zstring filterPhraseNew;
+        const Zchar* itFilterOrig = filterPhrase.begin();
 
-        auto replaceFullPaths = [&](Zstring& filterPhrase)
+        split2(filterPhrase, [](Zchar c) { return c == FILTER_ITEM_SEPARATOR || c == Zstr('\n'); }, //delimiters
+        [&](const ZstringView phrase)
         {
-            Zstring filterPhraseNew;
-            const Zchar* itFilterOrig = filterPhrase.begin();
-
-            split2(filterPhrase, [](Zchar c) { return c == FILTER_ITEM_SEPARATOR || c == Zstr('\n'); }, //delimiters
-            [&](ZstringView phrase)
+            const ZstringView phraseTrm = trimCpy(phrase);
+            if (!phraseTrm.empty())
             {
-                phrase = trimCpy(phrase);
-                if (!phrase.empty())
-                {
-                    const Zstring phraseNorm = normalizeForSearch(Zstring{phrase});
+                const Zstring phraseNorm = normalizeForSearch(Zstring{phraseTrm});
 
-                    for (const Zstring& pathNormPf : folderPathsPf)
-                        if (startsWith(phraseNorm, pathNormPf))
-                        {
-                            //emulate a "normalized afterFirst()":
-                            ptrdiff_t sepCount = std::count(pathNormPf.begin(), pathNormPf.end(), FILE_NAME_SEPARATOR);
-                            assert(sepCount > 0);
+                for (const Zstring& pathNormPf : folderPathsPf)
+                    if (startsWith(phraseNorm, pathNormPf))
+                    {
+                        //emulate a "normalized afterFirst()":
+                        ptrdiff_t sepCount = std::count(pathNormPf.begin(), pathNormPf.end(), FILE_NAME_SEPARATOR);
+                        assert(sepCount > 0);
 
-                            for (auto it = phrase.begin(); it != phrase.end(); ++it)
-                                if (*it == Zstr('/') ||
-                                    *it == Zstr('\\'))
-                                    if (--sepCount == 0)
-                                    {
-                                        const Zstring relPath(it, phrase.end()); //include first path separator
+                        for (auto it = phraseTrm.begin(); it != phraseTrm.end(); ++it)
+                            if (*it == Zstr('/') ||
+                                *it == Zstr('\\'))
+                                if (--sepCount == 0)
+                                {
+                                    const Zstring relPath(it, phraseTrm.end()); //include first path separator
 
-                                        filterPhraseNew.append(itFilterOrig, phrase.data());
-                                        filterPhraseNew += relPath;
-                                        itFilterOrig = phrase.data() + phrase.size();
+                                    filterPhraseNew.append(itFilterOrig, phraseTrm.data());
+                                    filterPhraseNew += relPath;
+                                    itFilterOrig = phraseTrm.data() + phraseTrm.size();
 
-                                        replacements.emplace_back(phrase, relPath);
-                                        return; //... to next block
-                                    }
-                            throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
-                        }
-                }
-            });
-            filterPhraseNew.append(itFilterOrig, filterPhrase.cend());
-
-            filterPhrase = filterPhraseNew;
-        };
-        replaceFullPaths(filterCfg.includeFilter);
-        replaceFullPaths(filterCfg.excludeFilter);
-
-        assert(!replacements.empty());
-        if (!replacements.empty())
-        {
-            std::wstring detailsMsg;
-            for (const auto& [from, to] : replacements)
-                detailsMsg += utfTo<std::wstring>(from) + L' ' + arrowRight + L' ' + utfTo<std::wstring>(to) + L'\n';
-            detailsMsg.pop_back();
-
-            switch (showConfirmationDialog(parent, DialogInfoType::info, PopupDialogCfg().
-                                           setMainInstructions(_("Each filter item must be a path relative to the selected folder pairs. The following changes are suggested:")).
-                                           setDetailInstructions(detailsMsg), _("&Change")))
-            {
-                case ConfirmationButton::accept: //change
-                    break;
-
-                case ConfirmationButton::cancel:
-                    return false;
+                                    replacements.emplace_back(phraseTrm, relPath);
+                                    return; //... to next block
+                                }
+                        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
+                    }
             }
+        });
+
+        if (itFilterOrig != filterPhrase.begin()) //perf!?
+        {
+            filterPhraseNew.append(itFilterOrig, filterPhrase.cend());
+            filterPhrase = std::move(filterPhraseNew);
+        }
+    };
+    replaceFullPaths(filterCfg.includeFilter);
+    replaceFullPaths(filterCfg.excludeFilter);
+
+    if (!replacements.empty())
+    {
+        std::wstring detailsMsg;
+        for (const auto& [from, to] : replacements)
+            if (to.empty())
+                detailsMsg += _("Remove:") + L' ' + utfTo<std::wstring>(from) + L'\n';
+            else
+                detailsMsg += utfTo<std::wstring>(from) + L' ' + arrowRight + L' ' + utfTo<std::wstring>(to) + L'\n';
+        detailsMsg.pop_back();
+
+        switch (showConfirmationDialog(parent, DialogInfoType::info, PopupDialogCfg().
+                                       setMainInstructions(_("Each filter item must be a path relative to the selected folder pairs. The following changes are suggested:")).
+                                       setDetailInstructions(detailsMsg), _("&Change")))
+        {
+            case ConfirmationButton::accept: //change
+                break;
+
+            case ConfirmationButton::cancel:
+                return false;
         }
     }
     return true;
@@ -223,6 +224,8 @@ private:
     void onCancel(wxCommandEvent& event) override { EndModal(static_cast<int>(ConfirmationButton::cancel)); }
     void onClose (wxCloseEvent&   event) override { EndModal(static_cast<int>(ConfirmationButton::cancel)); }
 
+    void onAddNotes(wxCommandEvent& event) override;
+
     void onLocalKeyEvent(wxKeyEvent& event);
     void onListBoxKeyEvent(wxKeyEvent& event) override;
     void onSelectFolderPair(wxCommandEvent& event) override;
@@ -269,8 +272,6 @@ private:
     void onFilterDefaultContextMouse(wxMouseEvent&   event) override { onFilterDefaultContext(static_cast<wxEvent&>(event)); }
     void onFilterDefaultContext(wxEvent& event);
 
-    void onFilterKeyEvent(wxKeyEvent& event);
-
     FilterConfig getFilterConfig() const;
     void setFilterConfig(const FilterConfig& filter);
 
@@ -323,6 +324,7 @@ private:
     void setSyncConfig(const SyncConfig* syncCfg);
 
     void updateSyncGui();
+    //-----------------------------------------------------
 
     //parameters with ownership NOT within GUI controls!
     SyncDirectionConfig directionCfg_;
@@ -368,6 +370,8 @@ private:
     int selectedPairIndexToShow_ = EMPTY_PAIR_INDEX_SELECTED;
     static const int EMPTY_PAIR_INDEX_SELECTED = -2;
 
+    bool showNotesPanel_ = false;
+
     const bool enableExtraFeatures_;
     const bool showMultipleCfgs_;
 
@@ -461,6 +465,7 @@ emailHistoryOut_(emailHistory),
 commandHistoryOut_(commandHistory),
 globalPairCfg_(globalPairCfg),
 localPairCfg_(localPairCfg),
+showNotesPanel_(!globalPairCfg.miscCfg.notes.empty()),
     enableExtraFeatures_(false),
 showMultipleCfgs_(showMultipleCfgs),
 globalLogFolderPhrase_(globalLogFolderPhrase)
@@ -469,23 +474,38 @@ globalLogFolderPhrase_(globalLogFolderPhrase)
 
     setStandardButtonLayout(*bSizerStdButtons, StdButtons().setAffirmative(m_buttonOkay).setCancel(m_buttonCancel));
 
+
+    setBitmapTextLabel(*m_buttonAddNotes, loadImage("notes", fastFromDIP(16)), m_buttonAddNotes->GetLabelText());
+
+    setImage(*m_bitmapNotes, loadImage("notes", fastFromDIP(20)));
+
+    //set reasonable default height for notes: simplistic algorithm neglecting line-wrap!
+    int notesRows = 1;
+    for (wchar_t c : trimCpy(globalPairCfg.miscCfg.notes))
+        if (c == L'\n')
+            ++notesRows;
+
+    double visibleRows = 5;
+    if (showNotesPanel_)
+        visibleRows = notesRows <= 10 ? notesRows : 10.5; //add half a row as visual hint
+    m_textCtrNotes->SetMinSize({-1, getTextCtrlHeight(*m_textCtrNotes, visibleRows)});
+
+
     m_notebook->SetPadding(wxSize(fastFromDIP(2), 0)); //height cannot be changed
 
     //fill image list to cope with wxNotebook image setting design desaster...
-    const int imgListSize = loadImage("options_compare_sicon").GetHeight();
+    const int imgListSize = fastFromDIP(16); //also required by GTK => don't use getDefaultMenuIconSize()
     auto imgList = std::make_unique<wxImageList>(imgListSize, imgListSize);
 
     auto addToImageList = [&](const wxImage& img)
     {
-        assert(img.GetWidth () <= imgListSize);
-        assert(img.GetHeight() <= imgListSize);
         imgList->Add(toScaledBitmap(img));
         imgList->Add(toScaledBitmap(greyScale(img)));
     };
     //add images in same sequence like ConfigTypeImage enum!!!
-    addToImageList(loadImage("options_compare_sicon"));
-    addToImageList(loadImage("options_filter_sicon"));
-    addToImageList(loadImage("options_sync_sicon"));
+    addToImageList(loadImage("options_compare", imgListSize));
+    addToImageList(loadImage("options_filter",  imgListSize));
+    addToImageList(loadImage("options_sync",    imgListSize));
     assert(imgList->GetImageCount() == static_cast<int>(ConfigTypeImage::syncGrey) + 1);
 
     m_notebook->AssignImageList(imgList.release()); //pass ownership
@@ -520,8 +540,8 @@ globalLogFolderPhrase_(globalLogFolderPhrase)
     const int scrollDelta = GetCharHeight();
     m_scrolledWindowPerf->SetScrollRate(scrollDelta, scrollDelta);
 
-    m_spinCtrlAutoRetryCount->SetMinSize({fastFromDIP(60), -1}); //Hack: set size (why does wxWindow::Size() not work?)
-    m_spinCtrlAutoRetryDelay->SetMinSize({fastFromDIP(60), -1}); //
+    setDefaultWidth(*m_spinCtrlAutoRetryCount);
+    setDefaultWidth(*m_spinCtrlAutoRetryDelay);
 
     //ignore invalid input for time shift control:
     wxTextValidator inputValidator(wxFILTER_DIGITS | wxFILTER_INCLUDE_CHAR_LIST);
@@ -533,8 +553,9 @@ globalLogFolderPhrase_(globalLogFolderPhrase)
 
     assert(!contains(m_buttonClear->GetLabel(), L"&C") && !contains(m_buttonClear->GetLabel(), L"&c")); //gazillionth wxWidgets bug on OS X: Command + C mistakenly hits "&C" access key!
 
-    m_textCtrlInclude->Bind(wxEVT_KEY_DOWN, [this](wxKeyEvent& event) { onFilterKeyEvent(event); });
-    m_textCtrlExclude->Bind(wxEVT_KEY_DOWN, [this](wxKeyEvent& event) { onFilterKeyEvent(event); });
+    setDefaultWidth(*m_spinCtrlMinSize);
+    setDefaultWidth(*m_spinCtrlMaxSize);
+    setDefaultWidth(*m_spinCtrlTimespan);
 
     m_staticTextFilterDescr->Wrap(fastFromDIP(450));
 
@@ -598,9 +619,9 @@ globalLogFolderPhrase_(globalLogFolderPhrase)
     add(VersioningStyle::timestampFolder, _("Time stamp") + L" [" + _("Folder") + L']', _("Move files into a time-stamped subfolder")).
     add(VersioningStyle::timestampFile,   _("Time stamp") + L" [" + _("File")   + L']', _("Append a time stamp to each file name"));
 
-    m_spinCtrlVersionMaxDays ->SetMinSize({fastFromDIP(60), -1}); //
-    m_spinCtrlVersionCountMin->SetMinSize({fastFromDIP(60), -1}); //Hack: set size (why does wxWindow::Size() not work?)
-    m_spinCtrlVersionCountMax->SetMinSize({fastFromDIP(60), -1}); //
+    setDefaultWidth(*m_spinCtrlVersionMaxDays );
+    setDefaultWidth(*m_spinCtrlVersionCountMin);
+    setDefaultWidth(*m_spinCtrlVersionCountMax);
 
     m_versioningFolderPath->setHistory(std::make_shared<HistoryList>(versioningFolderHistory, folderHistoryMax));
 
@@ -636,8 +657,9 @@ globalLogFolderPhrase_(globalLogFolderPhrase)
     m_comboBoxPostSyncCommand->SetHint(_("Example:") + L" systemctl poweroff");
 
     m_comboBoxPostSyncCommand->setHistory(commandHistory, commandHistoryMax);
-    //-----------------------------------------------------
 
+    //-----------------------------------------------------
+    //
     //enable dialog-specific key events
     Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) { onLocalKeyEvent(event); });
 
@@ -674,7 +696,10 @@ globalLogFolderPhrase_(globalLogFolderPhrase)
     selectFolderPairConfig(-1);
 
     GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
-    //=> works like a charm for GTK2 with window resizing problems and title bar corruption; e.g. Debian!!!
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
     Center(); //needs to be re-applied after a dialog size change!
 
     //keep stable sizer height: "two way" description is smaller than grid of sync directions
@@ -897,30 +922,13 @@ void ConfigDialog::onFilterDefaultContext(wxEvent& event)
     const FilterConfig defaultFilter = XmlGlobalSettings().defaultFilter;
 
     ContextMenu menu;
-    menu.addItem(_("&Save"),                 [&] { defaultFilterOut_ = activeCfg; updateFilterGui(); }, loadImage("cfg_save_sicon"), defaultFilterOut_ != activeCfg);
+    menu.addItem(_("&Save"),                 [&] { defaultFilterOut_ = activeCfg; updateFilterGui(); }, loadImage("cfg_save", getDefaultMenuIconSize()), defaultFilterOut_ != activeCfg);
     menu.addItem(_("&Load factory default"), [&] { setFilterConfig(defaultFilter); }, wxNullImage, activeCfg != defaultFilter);
 
     menu.popup(*m_bpButtonDefaultContext, {m_bpButtonDefaultContext->GetSize().x, 0});
 }
 
 
-void ConfigDialog::onFilterKeyEvent(wxKeyEvent& event)
-{
-    const int keyCode = event.GetKeyCode();
-
-    if (event.ControlDown())
-        switch (keyCode)
-        {
-            case 'A': //CTRL + A
-                if (auto textCtrl = dynamic_cast<wxTextCtrl*>(event.GetEventObject()))
-                    textCtrl->SelectAll();
-                return;
-        }
-
-    event.Skip();
-}
-
-
 FilterConfig ConfigDialog::getFilterConfig() const
 {
     const Zstring& includeFilter = utfTo<Zstring>(m_textCtrlInclude->GetValue());
@@ -1402,6 +1410,8 @@ MiscSyncConfig ConfigDialog::getMiscSyncOptions() const
     miscCfg.emailNotifyAddress = m_checkBoxSendEmail->GetValue() ? emailAddress : std::string();
     miscCfg.emailNotifyCondition = emailNotifyCondition_;
     //----------------------------------------------------------------------------
+    miscCfg.notes = trimCpy(utfTo<std::wstring>(m_textCtrNotes->GetValue()));
+
     return miscCfg;
 }
 
@@ -1421,7 +1431,7 @@ void ConfigDialog::setMiscSyncOptions(const MiscSyncConfig& miscCfg)
         for (int i = 0; i < rowsToCreate; ++i)
         {
             wxSpinCtrl* spinCtrlParallelOps = new wxSpinCtrl(m_scrolledWindowPerf, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 2000'000'000, 1);
-            spinCtrlParallelOps->SetMinSize({fastFromDIP(60), -1}); //Hack: set size (why does wxWindow::Size() not work?)
+            setDefaultWidth(*spinCtrlParallelOps);
             spinCtrlParallelOps->Enable(enableExtraFeatures_);
             fgSizerPerf->Add(spinCtrlParallelOps, 0, wxALIGN_CENTER_VERTICAL);
 
@@ -1469,6 +1479,8 @@ void ConfigDialog::setMiscSyncOptions(const MiscSyncConfig& miscCfg)
     m_comboBoxEmail->setValue(m_checkBoxSendEmail->GetValue() ? utfTo<Zstring>(miscCfg.emailNotifyAddress) : defaultEmail);
     emailNotifyCondition_ = miscCfg.emailNotifyCondition;
     //----------------------------------------------------------------------------
+    m_textCtrNotes->ChangeValue(utfTo<wxString>(miscCfg.notes));
+
     updateMiscGui();
 }
 
@@ -1538,6 +1550,10 @@ void ConfigDialog::updateMiscGui()
 
     m_panelSyncSettings->Refresh(); //removes a few artifacts when toggling email notifications
     m_panelLogfile     ->Refresh();//
+
+    //----------------------------------------------------------------------------
+    m_buttonAddNotes->Show(!showNotesPanel_);
+    m_panelNotes    ->Show(showNotesPanel_);
 }
 
 
@@ -1720,6 +1736,18 @@ bool ConfigDialog::unselectFolderPairConfig(bool validateParams)
 }
 
 
+void ConfigDialog::onAddNotes(wxCommandEvent& event)
+{
+    showNotesPanel_ = true;
+    updateMiscGui();
+
+    //=> enlarge dialog height!
+    GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
+
+    m_textCtrNotes->SetFocus();
+}
+
+
 void ConfigDialog::onOkay(wxCommandEvent& event)
 {
     if (!unselectFolderPairConfig(true /*validateParams*/))
