@@ -37,10 +37,10 @@ inline wxColor getColorPercentBackground() { return {0xf8, 0xf8, 0xf8}; }
 TreeView::TreeView(FolderComparison& folderCmp, const SortInfo& si) : folderCmp_(folderCmp), currentSort_(si)
 {
     //remove truly empty folder pairs as early as this: we want to distinguish single/multiple folder pair cases by looking at "folderCmp"
-    std::erase_if(folderCmp_, [](const std::shared_ptr<BaseFolderPair>& baseObj)
+    std::erase_if(folderCmp_, [](const SharedRef<BaseFolderPair>& baseObj)
     {
-        return AFS::isNullPath(baseObj->getAbstractPath<SelectSide::left >()) &&
-               AFS::isNullPath(baseObj->getAbstractPath<SelectSide::right>());
+        return AFS::isNullPath(baseObj.ref().getAbstractPath<SelectSide::left >()) &&
+               AFS::isNullPath(baseObj.ref().getAbstractPath<SelectSide::right>());
     });
 }
 
@@ -136,7 +136,7 @@ void TreeView::extractVisibleSubtree(ContainerObject& hierObj,  //in
 
 namespace
 {
-//generate nice percentage numbers which precisely add up to 100
+//generate "nice" percentage numbers which precisely add up to 100
 void calcPercentage(std::vector<std::pair<uint64_t, int*>>& workList)
 {
     uint64_t bytesTotal = 0;
@@ -382,11 +382,11 @@ void TreeView::updateView(Predicate pred)
     std::vector<RootNodeImpl> newView;
     newView.reserve(folderCmp_.size()); //avoid expensive reallocations!
 
-    for (const std::shared_ptr<BaseFolderPair>& baseObj : folderCmp_)
+    for (SharedRef<BaseFolderPair>& baseObj : folderCmp_)
     {
         newView.emplace_back();
         RootNodeImpl& root = newView.back();
-        this->extractVisibleSubtree(*baseObj, root, pred); //"this->" is bogus for a static method, but GCC screws this one up
+        this->extractVisibleSubtree(baseObj.ref(), root, pred); //"this->" is bogus for a static method, but GCC screws this one up
 
         //warning: the following lines are almost 1:1 copy from extractVisibleSubtree:
         //however we *cannot* reuse code here; this were only possible if we replaced "std::vector<RootNodeImpl>" with "Container"!
@@ -394,9 +394,9 @@ void TreeView::updateView(Predicate pred)
             newView.pop_back();
         else
         {
-            root.baseFolder = baseObj;
-            root.displayName = getShortDisplayNameForFolderPair(baseObj->getAbstractPath<SelectSide::left >(),
-                                                                baseObj->getAbstractPath<SelectSide::right>());
+            root.baseFolder = baseObj.ptr();
+            root.displayName = getShortDisplayNameForFolderPair(baseObj.ref().getAbstractPath<SelectSide::left >(),
+                                                                baseObj.ref().getAbstractPath<SelectSide::right>());
 
             this->compressNode(root); //"this->" required by two-pass lookup as enforced by GCC 4.7
         }
@@ -693,7 +693,7 @@ public:
         rootIcon_(loadImage("root_folder", widthNodeIcon_)),
         grid_(grid)
     {
-        grid.getMainWin().Bind(wxEVT_KEY_DOWN, [this](wxKeyEvent& event) { onKeyDown(event); });
+        grid.Bind(wxEVT_KEY_DOWN,                   [this](wxKeyEvent& event) { onKeyDown(event); });
         grid.Bind(EVENT_GRID_MOUSE_LEFT_DOWN,       [this](GridClickEvent& event) { onMouseLeft      (event); });
         grid.Bind(EVENT_GRID_MOUSE_LEFT_DOUBLE,     [this](GridClickEvent& event) { onMouseLeftDouble(event); });
         grid.Bind(EVENT_GRID_COL_LABEL_MOUSE_RIGHT, [this](GridLabelClickEvent& event) { onGridLabelContext  (event); });
@@ -798,6 +798,7 @@ private:
     enum class HoverAreaTree
     {
         node,
+        item,
     };
 
     void renderCell(wxDC& dc, const wxRect& rect, size_t row, ColumnType colType, bool enabled, bool selected, HoverArea rowHover) override
@@ -819,13 +820,6 @@ private:
         {
             if (std::unique_ptr<TreeView::Node> node = getDataView().getLine(row))
             {
-                ////clear first section:
-                //clearArea(dc, wxRect(rect.GetTopLeft(), wxSize(
-                //                         node->level_ * widthLevelStep_ + gapSize_ + //width
-                //                         (showPercentBar ? percentageBarWidth_ + 2 * gapSize_ : 0) + //
-                //                         widthNodeStatus_ + gapSize_ + widthNodeIcon + gapSize_, //
-                //                         rect.height)), wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
-
                 auto drawIcon = [&](wxImage icon, const wxRect& rectIcon, bool drawActive)
                 {
                     if (!drawActive)
@@ -851,7 +845,7 @@ private:
                     {
                         wxRect areaPerc(rectTmp.x, rectTmp.y + fastFromDIP(2), percentageBarWidth_, rectTmp.height - fastFromDIP(4));
                         //clear background
-                        drawInsetRectangle(dc, areaPerc, fastFromDIP(1), getColorPercentBorder(), getColorPercentBackground());
+                        drawFilledRectangle(dc, areaPerc, getColorPercentBackground(), getColorPercentBorder(), fastFromDIP(1));
                         areaPerc.Deflate(fastFromDIP(1));
 
                         //inner area
@@ -900,6 +894,9 @@ private:
 
                             drawIcon(nodeIcon, rectTmp, isActive);
 
+                            if (static_cast<HoverAreaTree>(rowHover) == HoverAreaTree::item)
+                                drawRectangleBorder(dc, rectTmp, *wxBLUE, fastFromDIP(1));
+
                             rectTmp.x     += widthNodeIcon_ + gapSize_;
                             rectTmp.width -= widthNodeIcon_ + gapSize_;
 
@@ -956,26 +953,18 @@ private:
 
     HoverArea getMouseHover(wxDC& dc, size_t row, ColumnType colType, int cellRelativePosX, int cellWidth) override
     {
-        switch (static_cast<ColumnTypeOverview>(colType))
-        {
-            case ColumnTypeOverview::folder:
-                if (std::unique_ptr<TreeView::Node> node = getDataView().getLine(row))
-                {
-                    const int tolerance = 2;
-                    const int nodeStatusXFirst = -tolerance + static_cast<int>(node->level_) * widthLevelStep_ + gapSize_ + (showPercentBar_ ? percentageBarWidth_ + 2 * gapSize_ : 0);
-                    const int nodeStatusXLast  = (nodeStatusXFirst + tolerance) + widthNodeStatus_ + tolerance;
-                    // -> synchronize renderCell() <-> getBestSize() <-> getMouseHover()
+        if (static_cast<ColumnTypeOverview>(colType) == ColumnTypeOverview::folder)
+            if (std::unique_ptr<TreeView::Node> node = getDataView().getLine(row))
+            {
+                const int nodeStatusXFirst = static_cast<int>(node->level_) * widthLevelStep_ + gapSize_ + (showPercentBar_ ? percentageBarWidth_ + 2 * gapSize_ : 0);
+                const int nodeStatusXLast  = nodeStatusXFirst + widthNodeStatus_;
+                // -> synchronize renderCell() <-> getBestSize() <-> getMouseHover()
 
-                    if (nodeStatusXFirst <= cellRelativePosX && cellRelativePosX < nodeStatusXLast)
-                        return static_cast<HoverArea>(HoverAreaTree::node);
-                }
-                break;
-
-            case ColumnTypeOverview::itemCount:
-            case ColumnTypeOverview::bytes:
-                break;
-        }
-        return HoverArea::none;
+                const int tolerance = fastFromDIP(5);
+                if (nodeStatusXFirst - tolerance <= cellRelativePosX && cellRelativePosX < nodeStatusXLast + tolerance)
+                    return static_cast<HoverArea>(HoverAreaTree::node);
+            }
+        return static_cast<HoverArea>(HoverAreaTree::item);
     }
 
     std::wstring getColumnLabel(ColumnType colType) const override
@@ -1007,6 +996,8 @@ private:
                         break;
                 }
                 break;
+            case HoverAreaTree::item:
+                break;
         }
         event.Skip();
     }
