@@ -26,7 +26,8 @@
 #include "../version/version.h"
 #include "small_dlgs.h"
 
-    #include <zen/symlink_target.h>
+    #include <gtk/gtk.h>
+    #include <wx/uilocale.h>
 
 
 using namespace zen;
@@ -91,8 +92,8 @@ std::wstring getIso639Language()
 {
     assert(runningOnMainThread()); //this function is not thread-safe: consider wxWidgets usage
 
-    std::wstring localeName(wxLocale::GetLanguageCanonicalName(wxLocale::GetSystemLanguage()));
-    localeName = beforeFirst(localeName, L'@', IfNotFoundReturn::all); //the locale may contain an @, e.g. "sr_RS@latin"; see wxLocale::InitLanguagesDB()
+    std::wstring localeName(wxUILocale::GetLanguageCanonicalName(wxUILocale::GetSystemLanguage()));
+    localeName = beforeFirst(localeName, L'@', IfNotFoundReturn::all); //the locale may contain an @, e.g. "sr_RS@latin"; see wxUILocale::InitLanguagesDB()
 
     if (!localeName.empty())
     {
@@ -111,8 +112,8 @@ std::wstring getIso3166Country()
 {
     assert(runningOnMainThread()); //this function is not thread-safe, consider wxWidgets usage
 
-    std::wstring localeName(wxLocale::GetLanguageCanonicalName(wxLocale::GetSystemLanguage()));
-    localeName = beforeFirst(localeName, L'@', IfNotFoundReturn::all); //the locale may contain an @, e.g. "sr_RS@latin"; see wxLocale::InitLanguagesDB()
+    std::wstring localeName(wxUILocale::GetLanguageCanonicalName(wxUILocale::GetSystemLanguage()));
+    localeName = beforeFirst(localeName, L'@', IfNotFoundReturn::all); //the locale may contain an @, e.g. "sr_RS@latin"; see wxUILocale::InitLanguagesDB()
 
     if (contains(localeName, L'_'))
     {
@@ -303,17 +304,17 @@ struct fff::UpdateCheckResultPrep
     std::vector<std::pair<std::string, std::string>> postParameters;
     std::optional<SysError> error;
 };
-std::shared_ptr<const UpdateCheckResultPrep> fff::automaticUpdateCheckPrepare(wxWindow& parent)
+SharedRef<const UpdateCheckResultPrep> fff::automaticUpdateCheckPrepare(wxWindow& parent)
 {
     assert(runningOnMainThread());
-    auto prep = std::make_shared<UpdateCheckResultPrep>();
+    auto prep = makeSharedRef<UpdateCheckResultPrep>();
     try
     {
-        prep->postParameters = geHttpPostParameters(parent); //throw SysError
+        prep.ref().postParameters = geHttpPostParameters(parent); //throw SysError
     }
     catch (const SysError& e)
     {
-        prep->error = e;
+        prep.ref().error = e;
     }
     return prep;
 }
@@ -325,33 +326,31 @@ struct fff::UpdateCheckResult
     bool internetIsAlive = false;
     std::optional<SysError> error;
 };
-std::shared_ptr<const UpdateCheckResult> fff::automaticUpdateCheckRunAsync(const UpdateCheckResultPrep* resultPrep)
+SharedRef<const UpdateCheckResult> fff::automaticUpdateCheckRunAsync(const UpdateCheckResultPrep& resultPrep)
 {
     //assert(!runningOnMainThread()); -> allow synchronous call, too
-    auto result = std::make_shared<UpdateCheckResult>();
+    auto result = makeSharedRef<UpdateCheckResult>();
     try
     {
-        if (resultPrep->error)
-            throw* resultPrep->error; //throw SysError
+        if (resultPrep.error)
+            throw* resultPrep.error; //throw SysError
 
-        result->onlineVersion = getOnlineVersion(resultPrep->postParameters); //throw SysError
-        result->internetIsAlive = true;
+        result.ref().onlineVersion = getOnlineVersion(resultPrep.postParameters); //throw SysError
+        result.ref().internetIsAlive = true;
     }
     catch (const SysError& e)
     {
-        result->error = e;
-        result->internetIsAlive = internetIsAlive();
+        result.ref().error = e;
+        result.ref().internetIsAlive = internetIsAlive();
     }
     return result;
 }
 
 
-void fff::automaticUpdateCheckEval(wxWindow& parent, time_t& lastUpdateCheck, std::string& lastOnlineVersion, const UpdateCheckResult* asyncResult)
+void fff::automaticUpdateCheckEval(wxWindow& parent, time_t& lastUpdateCheck, std::string& lastOnlineVersion, const UpdateCheckResult& result)
 {
     assert(runningOnMainThread());
 
-    const UpdateCheckResult& result = *asyncResult;
-
     if (!result.error)
     {
         lastUpdateCheck   = getVersionCheckCurrentTime();
@@ -377,7 +376,7 @@ void fff::automaticUpdateCheckEval(wxWindow& parent, time_t& lastUpdateCheck, st
                     break;
                 case ConfirmationButton2::accept2: //retry
                     automaticUpdateCheckEval(parent, lastUpdateCheck, lastOnlineVersion,
-                                             automaticUpdateCheckRunAsync(automaticUpdateCheckPrepare(parent).get()).get()); //note: retry via recursion!!!
+                                             automaticUpdateCheckRunAsync(automaticUpdateCheckPrepare(parent).ref()).ref()); //note: retry via recursion!!!
                     break;
                 case ConfirmationButton2::cancel:
                     break;
