@@ -207,7 +207,6 @@ private:
     void cleanup()
     {
         asyncStreamIn_->setReadError(std::make_exception_ptr(ThreadStopRequest()));
-        warn_static("log on error!")
     }
 
     std::shared_ptr<AsyncStreamBuffer> asyncStreamIn_ = std::make_shared<AsyncStreamBuffer>(HTTP_STREAM_BUFFER_SIZE);
@@ -499,7 +498,7 @@ bool zen::isValidEmail(const std::string_view& email)
         return false;
     //---------------------------------------------------------------------
 
-    //not going to parse and validate this!
+    //we're not going to parse and validate this!
     const bool quoted = (startsWith(local, '"') && endsWith(local, '"')) ||
                         contains(local, '\\'); //e.g. "t\@st@email.com"
     if (!quoted)
