@@ -13,10 +13,9 @@
 #include "app_main.h"
 #include "bitmap_button.h"
 #include "no_flicker.h"
-#include "font_size.h"
+#include "window_layout.h"
 #include "image_resources.h"
 #include "popup_dlg_generated.h"
-#include "std_button_layout.h"
 #include "taskbar.h"
 #include "window_tools.h"
 
@@ -92,6 +91,17 @@ void setBestInitialSize(wxRichTextCtrl& ctrl, const wxString& text, wxSize maxSi
 }
 
 
+int zen::getTextCtrlHeight(wxTextCtrl& ctrl, double rowCount)
+{
+    const int rowHeight =
+        ctrl.GetTextExtent(L"X").GetHeight();
+
+    return std::round(
+               2 +
+               rowHeight * rowCount);
+}
+
+
 class zen::StandardPopupDialog : public PopupDialogGenerated
 {
 public:
@@ -113,10 +123,10 @@ public:
                     case DialogInfoType::info:
                         break;
                     case DialogInfoType::warning:
-                        taskbar_->setStatus(Taskbar::STATUS_WARNING);
+                        taskbar_->setStatus(Taskbar::Status::warning);
                         break;
                     case DialogInfoType::error:
-                        taskbar_->setStatus(Taskbar::STATUS_ERROR);
+                        taskbar_->setStatus(Taskbar::Status::error);
                         break;
                 }
             }
@@ -273,12 +283,17 @@ public:
         //set std order after button visibility was set
         setStandardButtonLayout(*bSizerStdButtons, stdBtns);
 
-
         updateGui();
 
+
         GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize()
+#ifdef __WXGTK3__
+        Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+        Hide(); //avoid old position flash when Center() moves window (asynchronously?)
+#endif
         Center(); //needs to be re-applied after a dialog size change!
 
+
         Raise(); //[!] popup may be triggered by ffs_batch job running in the background!
 
         if (m_buttonAccept->IsEnabled())
