@@ -256,16 +256,26 @@ BinContainer unbufferedLoad(Function tryRead /*(void* buffer, size_t bytesToRead
 {
     static_assert(sizeof(typename BinContainer::value_type) == 1); //expect: bytes
     if (blockSize == 0)
-        throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
+        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
 
     BinContainer buf;
     for (;;)
     {
-        warn_static("don't need zero-initialization!")
-        buf.resize(buf.size() + blockSize);
+#ifndef ZEN_HAVE_RESIZE_AND_OVERWRITE
+#error include legacy_compiler.h!
+#endif
+#if ZEN_HAVE_RESIZE_AND_OVERWRITE //permature(?) perf optimization; avoid needless zero-initialization:
+        size_t bytesRead = 0;
+        buf.resize_and_overwrite(buf.size() + blockSize, [&, bufSizeOld = buf.size()](char* rawBuf, size_t /*rawBufSize: caveat: may be larger than what's requested*/)
+        {
+            bytesRead = tryRead(rawBuf + bufSizeOld, blockSize); //throw X; may return short; only 0 means EOF
+            return bufSizeOld + bytesRead;
+        });
+#else
+        buf.resize(buf.size() + blockSize); //needless zero-initialization!
         const size_t bytesRead = tryRead(buf.data() + buf.size() - blockSize, blockSize); //throw X; may return short; only 0 means EOF
         buf.resize(buf.size() - blockSize + bytesRead); //caveat: unsigned arithmetics
-
+#endif
         if (bytesRead == 0) //end of file
         {
             //caveat: memory consumption of returned string!
@@ -285,7 +295,7 @@ void unbufferedSave(const BinContainer& cont,
 {
     static_assert(sizeof(typename BinContainer::value_type) == 1); //expect: bytes
     if (blockSize == 0)
-        throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
+        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
 
     const size_t bufPosEnd = cont.size();
     size_t bufPos = 0;
@@ -311,7 +321,7 @@ void unbufferedStreamCopy(Function1 tryRead /*(void* buffer, size_t bytesToRead)
     blockSizeOut = std::bit_ceil(blockSizeOut);
 #endif
     if (blockSizeIn == 0 || blockSizeOut == 0)
-        throw std::logic_error("Contract violation! " + std::string(__FILE__) + ':' + numberTo<std::string>(__LINE__));
+        throw std::logic_error(std::string(__FILE__) + '[' + numberTo<std::string>(__LINE__) + "] Contract violation!");
 
     const size_t bufCapacity = blockSizeOut - 1 + blockSizeIn;
     const size_t alignment = ::sysconf(_SC_PAGESIZE); //-1 on error => posix_memalign() will fail
